//
//  BarGraph.h
//
//  Created by Robert on 11/10/2009.
//

#import <UIKit/UIKit.h>

// --------------------------------------------------------
// The Bar Object
// --------------------------------------------------------
@interface Bar : NSObject
{
@private
	NSString *	name;
	float		value;
	UIColor *	color;
	//bh
	NSString *	imgColor;
}

- (id) initWithName:(NSString*)_name AndColor:(UIColor*)_color AndImgColor:_imgColor AndValue:(float)_value;

@property (nonatomic, readonly) NSString *	name;
@property (nonatomic)           float		value;
@property (nonatomic, retain)   UIColor *	color;
@property (nonatomic, copy)		NSString *	imgColor;

@end

// --------------------------------------------------------
// Bar Graph View
//
// After Adding or updating values please use the function
// setNeedsDisplay to cause the view to re-draw.
// --------------------------------------------------------
@interface BarGraph : UIView 
{
@private
	
	// --------------------------------
	// Background Image :
	// will be rendered if not nil.
	// --------------------------------
	UIImage *		 backgroundImage;
	// --------------------------------
	
	// --------------------------------
	// Data Array :
	// Contains the bar objects.
	// --------------------------------
	NSMutableArray * values;
	
	NSString *		 verticalAxisName;
	NSString *		 horizontalAxisName;
	// --------------------------------
	
	// --------------------------------
	// Graph Indent :
	// How much away from the edges 
	// does the graph get rendered.
	// --------------------------------
	CGSize			indent;
	UIFont *		font;
	UIFont *		fontSmall;
	UIColor	*		fontColor;
	// --------------------------------
	
	// --------------------------------
	// line settings :
	// --------------------------------
	NSInteger		lineWidth;
	UIColor *		lineColor;
	// --------------------------------
	
	// --------------------------------
	// grid data
	// --------------------------------
	NSInteger		gridLineWidth;
	UIColor *		gridLineColor;
	// --------------------------------
	
	// --------------------------------
	// Fill data
	// --------------------------------
	UIColor *		fillColor;
	// --------------------------------
	
	// --------------------------------
	// bar layout values
	// --------------------------------
	NSInteger		barSeperator;
	NSInteger		barVerticalIndent;
	
	NSInteger		barOutlineWidth;
	UIColor *		barOutlineColor;
	
	float			barHighlightPercentage;
	UIColor *		barHighlightColor;

	CGPoint			barShadowOffset;
	UIColor *		barShadowColor;
	// --------------------------------
	
	// --------------------------------
	// Render Flags :
	// Indicate the parts of the bar
	// to render.
	// --------------------------------
	Boolean			drawBarHighlight;
	Boolean			drawBarOutline;
	Boolean			drawBarShadow;
	Boolean			drawBarPercentages;
	Boolean			drawBarNames;
	Boolean			drawGrid;
	Boolean			drawEdge;
	Boolean			drawLine;
	Boolean			drawFill;
	Boolean			drawLabels;
	Boolean			drawScale;
	// --------------------------------
	
	// bh 
	Boolean			comparisonChart;
	Boolean			useTwoLines;
	Boolean			showValueInside;
	Boolean			showPercengages;
	Boolean			drawRoundedFill;

}

@property (nonatomic, readonly)	NSInteger count;

@property (nonatomic, retain)	UIImage * backgroundImage;
@property (nonatomic, copy)		NSString * verticalAxisName;
@property (nonatomic, copy)		NSString * horizontalAxisName;

@property (nonatomic)			CGSize indent;
@property (nonatomic, retain)	UIFont * font;
@property (nonatomic, retain)	UIFont * fontSmall;
@property (nonatomic, retain)	UIColor	* fontColor;
@property (nonatomic)			NSInteger lineWidth;
@property (nonatomic, retain)	UIColor * lineColor;
@property (nonatomic)			NSInteger gridLineWidth;
@property (nonatomic, retain)	UIColor * gridLineColor;
@property (nonatomic, retain)	UIColor * fillColor;
@property (nonatomic)			NSInteger barSeperator;
@property (nonatomic)			NSInteger barVerticalIndent;
@property (nonatomic)			NSInteger barOutlineWidth;
@property (nonatomic, retain)	UIColor * barOutlineColor;
@property (nonatomic)			float barHighlightPercentage; // 0 - 1 value;
@property (nonatomic, retain)	UIColor * barHighlightColor;
@property (nonatomic)			CGPoint barShadowOffset;
@property (nonatomic, retain)	UIColor * barShadowColor;

@property (nonatomic)			Boolean drawBarHighlight;
@property (nonatomic)			Boolean drawBarOutline;
@property (nonatomic)			Boolean drawBarShadow;
@property (nonatomic)			Boolean drawBarPercentages;
@property (nonatomic)			Boolean drawBarNames;
@property (nonatomic)			Boolean drawGrid;
@property (nonatomic)			Boolean drawEdge;
@property (nonatomic)			Boolean drawLine;
@property (nonatomic)			Boolean	drawFill;
@property (nonatomic)			Boolean drawLabels;
@property (nonatomic)			Boolean drawScale;

@property (nonatomic)			Boolean comparisonChart;
@property (nonatomic)			Boolean useTwoLines;
@property (nonatomic)			Boolean showValueInside;
@property (nonatomic)			Boolean showPercengages;
@property (nonatomic)			Boolean	drawRoundedFill;

// remove all the current bars
- (void) removeAllBars;
- (void) removeBarAtIndex:(NSInteger)_index;
- (void) removeBarWithName:(NSString*)_name;
- (void) removeBar:(Bar*)_bar;

// add a bar
- (Bar*) addBarWithName:(NSString*)_name WithColor:(UIColor*)_color AndValue:(float)_value;
- (Bar*) addBarWithName:(NSString*)_name WithColor:(UIColor*)_color WithImgColor:(NSString*)_imgColor AndValue:(float)_value;

// retrieve the bar
- (Bar*) barAtIndex:(NSInteger)_index;
- (Bar*) barWithName:(NSString*)_name;

@end
// --------------------------------------------------------