//
//  BarGraph.m
//
//  Created by Robert on 11/10/2009.
//

#import "BarGraph.h"

// --------------------------------------------------------
// The Bar Object
// --------------------------------------------------------
@implementation Bar

#pragma mark ---------------------------------------------------------
#pragma mark === Properties  ===
#pragma mark ---------------------------------------------------------

@synthesize name;
@synthesize value;
@synthesize color;
// bh
@synthesize imgColor;

#pragma mark ---------------------------------------------------------
#pragma mark === End Properties  ===
#pragma mark ---------------------------------------------------------

#pragma mark ---------------------------------------------------------
#pragma mark === Constructor / Destructor Functions  ===
#pragma mark ---------------------------------------------------------

// -------------------------------------------------------------------
// initialization
// -------------------------------------------------------------------
- (id) initWithName:(NSString*)_name AndColor:(UIColor*)_color AndImgColor:_imgColor AndValue:(float)_value
{
	if ( self = [super init] )
	{
		name		= [_name copy];
		value		= _value;
		imgColor	= _imgColor;
		self.color	= _color;
	}
	
	return self;
}

// -------------------------------------------------------------------
//  destructor
// -------------------------------------------------------------------
- (void)dealloc 
{
	[color release];
    [super dealloc];
}

#pragma mark ---------------------------------------------------------
#pragma mark === End Constructor / Destructor Functions  ===
#pragma mark ---------------------------------------------------------

@end
// --------------------------------------------------------

// --------------------------------------------------------
// Bar Graph View
// --------------------------------------------------------
@implementation BarGraph

#pragma mark ---------------------------------------------------------
#pragma mark === Properties  ===
#pragma mark ---------------------------------------------------------

// -------------------------------------------------------------------
// Property : Return the number of bars contained
// -------------------------------------------------------------------
- (NSInteger) count
{
	return [values count];
}
 
@synthesize backgroundImage;
@synthesize verticalAxisName;
@synthesize horizontalAxisName;

@synthesize indent;
@synthesize font;
@synthesize fontSmall;
@synthesize fontColor;
@synthesize lineWidth;
@synthesize lineColor;
@synthesize gridLineWidth;
@synthesize gridLineColor;
@synthesize fillColor;
@synthesize barSeperator;
@synthesize barVerticalIndent;
@synthesize barOutlineWidth;
@synthesize barOutlineColor;
@synthesize barHighlightPercentage; 
@synthesize barHighlightColor;
@synthesize barShadowOffset;
@synthesize barShadowColor;

@synthesize drawBarHighlight;
@synthesize drawBarOutline;
@synthesize drawBarShadow;
@synthesize drawBarPercentages;
@synthesize drawBarNames;
@synthesize drawGrid;
@synthesize drawEdge;
@synthesize drawLine;
@synthesize drawFill;
@synthesize drawLabels;
@synthesize drawScale;

// bh
@synthesize comparisonChart;
@synthesize useTwoLines;
@synthesize showValueInside;
@synthesize showPercengages;
@synthesize drawRoundedFill;

#pragma mark ---------------------------------------------------------
#pragma mark === End Properties  ===
#pragma mark ---------------------------------------------------------

#pragma mark ---------------------------------------------------------
#pragma mark === Constructor / Destructor Functions  ===
#pragma mark ---------------------------------------------------------

// -------------------------------------------------------------------
//  initialisation
// -------------------------------------------------------------------
- (id)initWithFrame:(CGRect)frame 
{
    if (self = [super initWithFrame:frame]) 
	{
        // Initialization code
		values = [[NSMutableArray alloc] initWithCapacity:10];

    // bh
		
		indent.width			= 5;
		indent.height			= 2;
		
		font					= [[UIFont fontWithName:@"Helvetica" size:12] retain];
		fontSmall				= [[UIFont fontWithName:@"Helvetica" size:10] retain];
		fontColor				= [[UIColor blackColor] retain];
		
		lineWidth				= 2;
		lineColor				= [[UIColor blackColor] retain];
		
		gridLineWidth			= 1;
		gridLineColor			= [[UIColor blackColor] retain];
		
		fillColor				= [[UIColor colorWithRed:0 green:0 blue:0 alpha:0.1f] retain];
		
		barSeperator			= 5;
		barVerticalIndent		= 2;
		barOutlineWidth			= 1;
		barOutlineColor			= [[UIColor blackColor] retain];
		
		barHighlightPercentage	= 0.2f;
		barHighlightColor		= [[UIColor colorWithWhite:1 alpha:0.3f] retain]; 
		
		barShadowColor			= [[UIColor colorWithRed:0 green:0 blue:0 alpha:0.5f] retain];
		barShadowOffset			= CGPointMake( 2, 2 );
		
		drawLine				= TRUE;
		drawGrid				= TRUE;
		drawEdge				= TRUE;
		drawFill				= TRUE;
		drawLabels				= TRUE;
		drawScale				= TRUE;
		drawBarHighlight		= TRUE;
		drawBarOutline			= TRUE;
		drawBarShadow			= TRUE;
		drawBarPercentages		= TRUE;
		drawBarNames			= TRUE;
			
		// bh
		comparisonChart			= FALSE;
		useTwoLines				= FALSE;
		showValueInside			= TRUE;
		showPercengages			= TRUE;
		drawRoundedFill			= FALSE;

	}
    return self;
}

// -------------------------------------------------------------------
// Initialisation
// -------------------------------------------------------------------
- (id)initWithCoder:(NSCoder*)coder 
{	
	if ( self = [super initWithCoder:coder] )
	{
		// Initialization code
		values = [[NSMutableArray alloc] initWithCapacity:10];
		
		// ------------------------------
		// test data
		// ------------------------------
		/*
		[values addObject:[[Bar alloc] initWithName:@"one"   AndColor:[UIColor redColor] AndValue:5]];
		[values addObject:[[Bar alloc] initWithName:@"two"   AndColor:[UIColor redColor] AndValue:15]];
		[values addObject:[[Bar alloc] initWithName:@"three" AndColor:[UIColor redColor] AndValue:10]];
		[values addObject:[[Bar alloc] initWithName:@"three" AndColor:[UIColor redColor] AndValue:-10]];
		[values addObject:[[Bar alloc] initWithName:@"three" AndColor:[UIColor redColor] AndValue:-15]];
		
		verticalAxisName   = @"Vertical Axis";
		horizontalAxisName = @"Horizontal Axis";
		*/
		// ------------------------------
		
		indent.width			= 5;
		indent.height			= 2;
		
		font					= [[UIFont fontWithName:@"Helvetica" size:12] retain];
		fontSmall				= [[UIFont fontWithName:@"Helvetica" size:10] retain];
		fontColor				= [[UIColor blackColor] retain];
		
		lineWidth				= 2;
		lineColor				= [[UIColor blackColor] retain];
		
		gridLineWidth			= 1;
		gridLineColor			= [[UIColor blackColor] retain];
		
		fillColor				= [[UIColor colorWithRed:0 green:0 blue:0 alpha:0.1f] retain];
		
		barSeperator			= 5;
		barVerticalIndent		= 2;
		barOutlineWidth			= 1;
		barOutlineColor			= [[UIColor blackColor] retain];
		
		barHighlightPercentage	= 0.2f;
		barHighlightColor		= [[UIColor colorWithWhite:1 alpha:0.3f] retain]; 
		
		barShadowColor			= [[UIColor colorWithRed:0 green:0 blue:0 alpha:0.5f] retain];
		barShadowOffset			= CGPointMake( 2, 2 );
		
		drawLine				= TRUE;
		drawGrid				= TRUE;
		drawEdge				= TRUE;
		drawFill				= TRUE;
		drawLabels				= TRUE;
		drawScale				= TRUE;
		drawBarHighlight		= TRUE;
		drawBarOutline			= TRUE;
		drawBarShadow			= TRUE;
		drawBarPercentages		= TRUE;
		drawBarNames			= TRUE;

		// bh
		comparisonChart			= FALSE;
		useTwoLines				= FALSE;
		showValueInside			= TRUE;
		showPercengages			= TRUE;
		drawRoundedFill			= FALSE;
	}
	return self;
}

// -------------------------------------------------------------------
//  destructor
// -------------------------------------------------------------------
- (void)dealloc 
{
	[values release];
	[font release];
	[fontSmall release];
	[fillColor release];
	[lineColor release];
	[gridLineColor release];
	[barHighlightColor release];
	[barOutlineColor release];
	[barShadowColor release];
	
    [super dealloc];
}

#pragma mark ---------------------------------------------------------
#pragma mark === End Constructor / Destructor Functions  ===
#pragma mark ---------------------------------------------------------

#pragma mark ---------------------------------------------------------
#pragma mark === View Functions  ===
#pragma mark ---------------------------------------------------------

// -------------------------------------------------------------------
// Custom Draw Code
// -------------------------------------------------------------------
- (void)drawRect:(CGRect)rect 
{
    [super drawRect:rect];

	// ----------------------------------------
	// Grab the context
	// ----------------------------------------
	CGContextRef context = UIGraphicsGetCurrentContext();
	const CGAffineTransform textTransform = CGAffineTransformMake( 1.0, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f );
	const CGAffineTransform rotTextTransform = CGAffineTransformRotate( textTransform, 1.57079633 );
	// ----------------------------------------
		
	// ----------------------------------------
	// draw the background image
	// ----------------------------------------
	if ( backgroundImage )
	{
		CGContextDrawImage( context, rect, [backgroundImage CGImage] );
	}
	// ----------------------------------------
	
	// ----------------------------------------
	// Calculate the min and max values
	// ----------------------------------------
	float minValue		= 1000000000.0f;
	float maxValue		= -1000000000.0f;
	float total			= 0.0;
	float totalB		= 0.0;
	int i				= 0;
	for (Bar * obj in values) 
	{
		float value = [obj value];
		total += value;
		if (comparisonChart && i%2) totalB += value;
		if (value > maxValue) maxValue = value;
		if (value < minValue) minValue = value;
		i++;
	}
	
	maxValue += 5;
	if (maxValue > 100)
		maxValue += 20;
	if ( [values count] == 0 )
	{
		minValue = 0;
		maxValue = 100.0f;
	}
	// ----------------------------------------
	
	// ----------------------------------------
	// Work out the bar draw area
	// ----------------------------------------	
	CGRect area     = CGRectMake( indent.width, 
								  indent.height, 
								  rect.size.width - ( indent.width * 2 ), 
								  rect.size.height - ( indent.height * 2 ) );
	area.size.height -= 10;
	// ----------------------------------------
	
	// ----------------------------------------
	// Draw the axis labels
	// ----------------------------------------
	if ( drawLabels )
	{
		// Set the Context Font the custom one				
		CGContextSelectFont(context, [font.fontName UTF8String], [font pointSize], kCGEncodingMacRoman);
		
		//The following code is for Drawing Text
		CGContextSetTextDrawingMode (context, kCGTextFill); 
		
		// set the font color
		CGContextSetFillColorWithColor(context, [fontColor CGColor]);
				
		// if we have a horizontal axis name
		// render it.
		if ( ( horizontalAxisName ) && 
			( [horizontalAxisName length] ) )
		{
			CGSize stringSize = [horizontalAxisName sizeWithFont:font];
			
			area.size.height -= [font pointSize];
			
			int x   = area.origin.x + ( area.size.width * 0.5f ) - ( stringSize.width * 0.5f );
			int y   = area.origin.y + area.size.height + [font pointSize];
		
			// flip the context text matrix as fonts are rendered upsode down.
			CGContextSetTextMatrix(context, textTransform);
			
			// draw the text.
			CGContextShowTextAtPoint(context, x, y, [horizontalAxisName UTF8String], horizontalAxisName.length);
		}
		
		// if we have a vertical axis name
		// render it
		if ( ( verticalAxisName ) &&
			 ( [verticalAxisName length] ) )
		{
			CGSize stringSize = [verticalAxisName sizeWithFont:font];
			int addition      = 4;
			
			area.origin.x    += [font pointSize] + addition;
			area.size.width  -= [font pointSize] + addition;
			
			int x   = area.origin.x - addition;
			int y   = area.origin.y + ( area.size.height* 0.5f ) + ( stringSize.width * 0.5f );

			// flip the context text matrix as fonts are rendered upsode down.
			// rotate round 90 dgrees
			CGContextSetTextMatrix(context, rotTextTransform);
			
			// draw the text.
			CGContextShowTextAtPoint(context, x, y, [verticalAxisName UTF8String], verticalAxisName.length);
		}
	}
	// ----------------------------------------

	// ----------------------------------------
	// shift the area for the scale to be 
	// rendered this will be done after
	// the center line has been found.
	// ----------------------------------------
	if ( drawScale )
	{
		NSString * zero = @"0";
		CGSize size = [zero sizeWithFont:font];
		
		int addition      = 2;
		if ( size.width > [font pointSize] )
		{
			area.origin.x    += size.width + addition;
			area.size.width  -= size.width + addition;
		}
		else
		{
			area.origin.x    += [font pointSize] + addition;
			area.size.width  -= [font pointSize] + addition;
		}		
	}
	// ----------------------------------------
	
	// ----------------------------------------
	// draw the bar names
	// ----------------------------------------
	if ( drawBarNames )
	{
		// shift ther Bar render area in,
		// this is so we can fit the names in.
		int space = 1;
		if (useTwoLines) space = 2;

		area.size.height -= space*[font pointSize];
		
		// reduce the size of the names a little to differentiate them from the
		// labels
		int width	   = ( lineWidth / 2 );
		int pointX     = area.origin.x + width;
		int pointY	   = area.origin.y + area.size.height;
		// int step       = 60; // ( [values count] ) ? ( area.size.width - width ) / [values count] : 0;
		int step       = ( [values count] ) ? ( area.size.width - width ) / [values count] : 0;
		if (comparisonChart) step *= 2;
		
		// flip the context text matrix as fonts are rendered upsode down.
		CGAffineTransform textTransform = CGAffineTransformMake( 1.0, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f );
		CGContextSetTextMatrix(context, textTransform);
		
		// Set the Context Font the custom one				
		CGContextSelectFont(context, [fontSmall.fontName UTF8String], fontSmall.pointSize, kCGEncodingMacRoman);
		
		//The following code is for Drawing Text
		CGContextSetTextDrawingMode (context, kCGTextFill); 
		
		// set the font color
		CGContextSetFillColorWithColor(context, [[UIColor blackColor] CGColor]);
		
		// draw the images
		int i = 1;
		int j = 1;
		for ( Bar * obj in values )
		{	

			// Measure the string size.
			CGSize stringSize = [obj.name sizeWithFont:fontSmall];
			
			// work out the point to render the text.
			NSInteger  x = pointX + ( step * 0.5f ) - ( stringSize.width * 0.5f );
			NSInteger  y = ((useTwoLines) ? ((j%2) ? 0 : 10 ) : 0) + pointY + fontSmall.pointSize;
			// NSLog(@"? x: %d y: %d ? %d i: %d j: %d", x, y, step, i, j);
			if (comparisonChart) x += 5;
			
			// draw the text.
			if (comparisonChart) {
			if (i%2) {
				j++;
				CGContextShowTextAtPoint(context, x, y, [obj.name UTF8String], obj.name.length);
				pointX += step;
			}
				i++;
			} else {
				CGContextShowTextAtPoint(context, x, y, [obj.name UTF8String], obj.name.length);
				pointX += step;
				if (useTwoLines) j++;
			}


		}
	}
	// ----------------------------------------
	
	// ----------------------------------------
	// Work out were the horazontal line should
	// sit.
	// ----------------------------------------
	float lineHeight = area.origin.y + ( ( minValue >= 0.0f ) ? area.size.height - ((lineWidth > 1) ? lineWidth * 0.5f : 0) : area.size.height * 0.5f ); // move the line to the center if we have negative values
	// ----------------------------------------
	
	// ----------------------------------------
	// draw the scale values
	// ----------------------------------------
	if ( drawScale )
	{
		// Set the Context Font the custom one				
		CGContextSelectFont(context, [font.fontName UTF8String], [font pointSize] - 2, kCGEncodingMacRoman);
		
		//The following code is for Drawing Text
		// CGContextSetTextDrawingMode (context, kCGTextFill); 
		
		// set the font color
		CGContextSetFillColorWithColor(context, [fontColor CGColor]);
		
		// --------------------------------------------
		// render the zero mark
		// --------------------------------------------
		{
			NSString * zero = @"0";
			CGSize size = [zero sizeWithFont:font];
			
			int x   = area.origin.x - ( size.width + 2 );
			int y   = lineHeight + ( [font pointSize] * 0.5f );
			
			// flip the context text matrix as fonts are rendered upsode down.
			CGContextSetTextMatrix(context, textTransform);
			
			// draw the text.
			CGContextShowTextAtPoint(context, x, y, [zero UTF8String], zero.length);
		}
		// --------------------------------------------
		
		// --------------------------------------------
		// render the max value
		// --------------------------------------------
		{
			NSString * strMax = (maxValue > 1) ?  [NSString stringWithFormat:@"%d", (int)maxValue] : [NSString stringWithFormat:@"%f", maxValue];
			CGSize size = [strMax sizeWithFont:font];
		
			// int x   = area.origin.x - 2;
			int x   = area.origin.x - ( size.width + 2 );
			// int y   = area.origin.y + size.width;
			int y   = area.origin.y + size.width - 5;
			
			// flip the context text matrix as fonts are rendered upsode down.
			// CGContextSetTextMatrix(context, rotTextTransform);
		
			// draw the text.
			// ???? ? ? ? ? ???????
			CGContextShowTextAtPoint(context, x, y, [strMax UTF8String], strMax.length);
		}
		// --------------------------------------------
		
		// --------------------------------------------
		// render the min value if less that zero
		// --------------------------------------------
		if ( minValue < 0 )
		{
			NSString * strMax = (fabs(minValue) > 1) ?  [NSString stringWithFormat:@"%d", (int)minValue] : [NSString stringWithFormat:@"%f", minValue];
			
			int x   = area.origin.x - 2;
			int y   = area.origin.y + area.size.height;
			
			// flip the context text matrix as fonts are rendered upsode down.
			CGContextSetTextMatrix(context, rotTextTransform);
			
			// draw the text.
			CGContextShowTextAtPoint(context, x, y, [strMax UTF8String], strMax.length);
		}		
		// --------------------------------------------
	}
	// ----------------------------------------
	
	// ----------------------------------------
	// draw the fill area this is the bit 
	// directly behind the graph
	// ----------------------------------------
	if ( drawFill )
	{
		CGContextSetAllowsAntialiasing(context, NO);
		
		// ------------------------------------------------------------
		// Draw the bar rect
		// ------------------------------------------------------------
		CGContextSetFillColorWithColor(context,[fillColor CGColor]);
		CGContextFillRect(context, area);
		// ------------------------------------------------------------
		
		CGContextSetAllowsAntialiasing(context, YES);
	}

	if ( drawRoundedFill )
	{
		int radius = 10;

		/*
		CGContextSaveGState(context);
		CGContextBeginPath(context); // gridLineColor [UIColor redColor]
		UIColor * tmpColor = [gridLineColor colorWithAlphaComponent:0.3];
		CGContextSetFillColor(context, CGColorGetComponents( [tmpColor CGColor]));
		CGContextMoveToPoint(context, CGRectGetMinX(area) + radius +gridLineWidth+3, 3+CGRectGetMinY(area)-gridLineWidth);
		
		CGContextAddArc(context, CGRectGetMaxX(area) - radius+gridLineWidth+3, 3+CGRectGetMinY(area) + radius-gridLineWidth, radius, 3 * M_PI / 2, 0, 0);
		CGContextAddArc(context, CGRectGetMaxX(area) - radius+gridLineWidth+3, 3+/ *CGRectGetMaxY(area)* /area.size.height - radius+gridLineWidth, radius, 0, M_PI / 2, 0);
		CGContextAddArc(context, CGRectGetMinX(area) + radius-gridLineWidth+3, 3+/ *CGRectGetMaxY(area) * /area.size.height - radius+gridLineWidth, radius, M_PI / 2, M_PI, 0);
		CGContextAddArc(context, CGRectGetMinX(area) + radius-gridLineWidth+3, 3+CGRectGetMinY(area) + radius-gridLineWidth, radius, M_PI, 3 * M_PI / 2, 0);	
		CGContextClosePath(context);
		CGContextFillPath(context);

		CGContextSaveGState(context);
		CGContextBeginPath(context); // gridLineColor [UIColor redColor]
		tmpColor = [gridLineColor colorWithAlphaComponent:0.6];
		CGContextSetFillColor(context, CGColorGetComponents( [tmpColor CGColor]));

		CGContextMoveToPoint(context, CGRectGetMinX(area) + radius +gridLineWidth+2, 2+CGRectGetMinY(area)-gridLineWidth);
		CGContextAddArc(context, CGRectGetMaxX(area) - radius+gridLineWidth+2, 2+CGRectGetMinY(area) + radius-gridLineWidth, radius, 3 * M_PI / 2, 0, 0);
		CGContextAddArc(context, CGRectGetMaxX(area) - radius+gridLineWidth+2, 2+/ *CGRectGetMaxY(area)* /area.size.height - radius+gridLineWidth, radius, 0, M_PI / 2, 0);
		CGContextAddArc(context, CGRectGetMinX(area) + radius-gridLineWidth+2, 2+/ *CGRectGetMaxY(area)* /area.size.height - radius+gridLineWidth, radius, M_PI / 2, M_PI, 0);
		CGContextAddArc(context, CGRectGetMinX(area) + radius-gridLineWidth+2, 2+CGRectGetMinY(area) + radius-gridLineWidth, radius, M_PI, 3 * M_PI / 2, 0);	
		CGContextClosePath(context);
		CGContextFillPath(context);


		CGContextBeginPath(context);
		tmpColor = [gridLineColor colorWithAlphaComponent:0.8];
		CGContextSetFillColor(context, CGColorGetComponents( [tmpColor CGColor]));

		CGContextMoveToPoint(context, CGRectGetMinX(area) + radius +gridLineWidth+1, 1+CGRectGetMinY(area)-gridLineWidth);
		CGContextAddArc(context, CGRectGetMaxX(area) - radius+gridLineWidth+1, 1+CGRectGetMinY(area) + radius-gridLineWidth, radius, 3 * M_PI / 2, 0, 0);
		CGContextAddArc(context, CGRectGetMaxX(area) - radius+gridLineWidth+1, 1+/ *CGRectGetMaxY(area)* /area.size.height - radius+gridLineWidth, radius, 0, M_PI / 2, 0);
		CGContextAddArc(context, CGRectGetMinX(area) + radius-gridLineWidth+1, 1+/ *CGRectGetMaxY(area)* /area.size.height - radius+gridLineWidth, radius, M_PI / 2, M_PI, 0);
		CGContextAddArc(context, CGRectGetMinX(area) + radius-gridLineWidth+1, 1+CGRectGetMinY(area) + radius-gridLineWidth, radius, M_PI, 3 * M_PI / 2, 0);	
		CGContextClosePath(context);
		CGContextFillPath(context);
		CGContextRestoreGState(context);
*/
		/* START OUTLINE */
		CGContextSaveGState(context);
		CGContextBeginPath(context); // gridLineColor [UIColor redColor]
		CGContextSetFillColor(context, CGColorGetComponents( [gridLineColor CGColor]));
		CGContextMoveToPoint(context, CGRectGetMinX(area) + radius +gridLineWidth, CGRectGetMinY(area)-gridLineWidth);
		CGContextAddArc(context, CGRectGetMaxX(area) - radius+gridLineWidth, CGRectGetMinY(area) + radius-gridLineWidth, radius, 3 * M_PI / 2, 0, 0);
		CGContextAddArc(context, CGRectGetMaxX(area) - radius+gridLineWidth, /*CGRectGetMaxY(area)*/area.size.height - radius+gridLineWidth, radius, 0, M_PI / 2, 0);
		CGContextAddArc(context, CGRectGetMinX(area) + radius-gridLineWidth, /*CGRectGetMaxY(area)*/area.size.height - radius+gridLineWidth, radius, M_PI / 2, M_PI, 0);
		CGContextAddArc(context, CGRectGetMinX(area) + radius-gridLineWidth, CGRectGetMinY(area) + radius-gridLineWidth, radius, M_PI, 3 * M_PI / 2, 0);	
		CGContextClosePath(context);
		CGContextFillPath(context);
		CGContextRestoreGState(context);
		/* END OUTLINE */

		CGContextBeginPath(context);
		CGContextSetFillColor(context, CGColorGetComponents( [fillColor CGColor]));
		CGContextMoveToPoint(context, CGRectGetMinX(area) + radius, CGRectGetMinY(area));
		CGContextAddArc(context, CGRectGetMaxX(area) - radius, CGRectGetMinY(area) + radius, radius, 3 * M_PI / 2, 0, 0);
		CGContextAddArc(context, CGRectGetMaxX(area) - radius, /*CGRectGetMaxY(area)*/area.size.height - radius, radius, 0, M_PI / 2, 0);
		CGContextAddArc(context, CGRectGetMinX(area) + radius, /*CGRectGetMaxY(area)*/area.size.height - radius, radius, M_PI / 2, M_PI, 0);
		CGContextAddArc(context, CGRectGetMinX(area) + radius, CGRectGetMinY(area) + radius, radius, M_PI, 3 * M_PI / 2, 0);	
		CGContextClosePath(context);
		CGContextFillPath(context);

	}
	
	// ----------------------------------------
	
	// ----------------------------------------
	// draw a grid
	// ----------------------------------------
	if ( drawGrid )
	{
		// Work out the edge of the graph
		CGPoint min	 = area.origin;
		CGPoint max	 = CGPointMake( area.origin.x + area.size.width, area.origin.y + area.size.height );
		
		CGContextSetAllowsAntialiasing(context, NO);
		
		// start the drawing
		CGContextBeginPath(context);
		
		CGContextSetStrokeColorWithColor(context, [gridLineColor CGColor]);
		
		// set the line width
		CGContextSetLineWidth(context, gridLineWidth);
		
		CGContextMoveToPoint(context, min.x, min.y );
		if (!drawRoundedFill)
		CGContextAddLineToPoint(context, min.x, max.y );
		
		int lines = 10;
		float height = min.y;
		float stepHeight = area.size.height / lines;
		for ( int i=0; i<lines-1; i++ )
		{
			height += stepHeight;
			CGContextMoveToPoint(context, min.x, height );
			CGContextAddLineToPoint(context, max.x, height );
		}	
		
		CGContextDrawPath(context, kCGPathStroke);
		
		CGContextSetAllowsAntialiasing(context, YES);
	}

	if ( drawEdge )
	{
		// Work out the edge of the graph
		CGPoint min	 = area.origin;
		CGPoint max	 = CGPointMake( area.origin.x + area.size.width, area.origin.y + area.size.height );
				
		CGContextSetAllowsAntialiasing(context, NO);
		
		// start the drawing
		CGContextBeginPath(context);
		
		CGContextSetStrokeColorWithColor(context, [gridLineColor CGColor]);
		
		// set the line width
		CGContextSetLineWidth(context, gridLineWidth);
				
		CGContextMoveToPoint(context, min.x, min.y );
		CGContextAddLineToPoint(context, min.x, max.y );
		CGContextAddLineToPoint(context, max.x, max.y );
		CGContextAddLineToPoint(context, max.x, min.y );
		CGContextAddLineToPoint(context, min.x, min.y );
		
		CGContextDrawPath(context, kCGPathStroke);
		
		CGContextSetAllowsAntialiasing(context, YES);

		// bh shadow
		//int offsetX = ( barShadowOffset.x > 0.0f ) ? 1 : -1;
		//int additionY = barShadowOffset.y;
		//int additionX = barShadowOffset.x - offsetX;
		// CGRect barShadow = CGRectMake(max.x, min.y, max.x+additionX, max.y+additionY );
		// CGContextSetFillColorWithColor(context, [UIColor greenColor].CGColor); // [barShadowColor CGColor]);
		// CGContextFillRect(context, barShadow);

		/*
		for (int i=0;i<3;i++) {
			float alphaColor = (3-i)/4;
			UIColor * tmpColor = [[UIColor blueColor] colorWithAlphaComponent:alphaColor];
			CGRect barShadow = CGRectMake(max.x+(1+(3*i)), min.y+i, max.x+(3+(i*2)), max.y+i );
			CGContextSetFillColorWithColor(context, tmpColor.CGColor); // [barShadowColor CGColor]);
			CGContextFillRect(context, barShadow);
		}


		
		/ *

		CGGradientRef myGradient;
		CGColorSpaceRef myColorspace;
		
		size_t num_locations = 2;
		CGFloat locations[2] = { 0.0, 1.0 };
		CGFloat components[8] = {  0.92, 0.92, 0.92, 1.0, 0.82, 0.82, 0.82, 0.4 };
		
		// myColorspace = CGColorSpaceCreateWithName(kCGColorSpaceGenericRGB);
		myColorspace  = CGColorSpaceCreateDeviceRGB();
		myGradient = CGGradientCreateWithColorComponents (myColorspace, components, locations, num_locations);
		
		CGPoint myStartPoint, myEndPoint;
		myStartPoint.x = 0.0;
		myStartPoint.y = 0.0;
		myEndPoint.x = 30.0;
		myEndPoint.y = 0.0;
		CGContextClip(context);
		CGContextDrawLinearGradient (context, myGradient, myStartPoint, myEndPoint, 0);
		
		CGPoint startPoint, endPoint;
		// here x1, y1 and x2, y2 should cover the area that you want to apply gradient on
		startPoint = CGPointMake(50, 50); 
		endPoint = CGPointMake(100, 100);
		// set the clipping path
		CGContextClip(context);
		// CGContextDrawLinearGradient(context, myGradient, startPoint, endPoint, 0);
		
		
		/ *
		CGContextRef ctx = UIGraphicsGetCurrentContext();
		CGColorSpaceRef colorSpace  = CGColorSpaceCreateDeviceRGB();
		/ *CGFloat components [12] = {1.0, 0.0, 0.0, 1.0,
			0.0, 0.0, 0.0, 1.0,
			0.0, 1.0, 0.0, 1.0};* 
		// CGFloat components [8] = { 0.0, 0.0, 0.0, 0.0, 1.0, 1.0, 1.0, 1.0};
		// CGFloat locations[2] = {0.0, 1.0};
		
		CGGradientRef gradient =
        CGGradientCreateWithColorComponents(colorSpace,
											components,
											locations,
											(size_t)3);
		CGContextDrawLinearGradient(ctx,
									gradient,
									CGPointMake(0, 0),
									CGPointMake(30, 30),
									(CGGradientDrawingOptions)NULL);
		CGColorSpaceRelease(colorSpace);
		*/
		// bh shadow
		
	}
	// ----------------------------------------
	
	// ----------------------------------------
	// draw the bars
	// ----------------------------------------
	{
		// turn off antialasing
		CGContextSetAllowsAntialiasing(context, NO);
		
		NSInteger count  = [values count]; 	
		int seperator  = barSeperator;
		int pointX     = area.origin.x + ( lineWidth / 2 ) + seperator;
		int height     = ( ( area.size.height - lineWidth ) * ( ( minValue >= 0.0f ) ? 1.0f : 0.5f ) ) - barVerticalIndent; // last 2 brings it in from the edge
		// int width	   = 55; // ( count ) ? ( area.size.width - ( ( seperator * ( count + 1 ) ) + ( lineWidth / 2 ) ) ) / count : 0;
		int width	   = ( count ) ? ( area.size.width - ( ( seperator * ( count + 1 ) ) + ( lineWidth / 2 ) ) ) / count : 0;
		
		// int outlineWidth; = barOutlineWidth;
		// int outlineHalfWidth; = (outlineWidth > 1) ? outlineWidth * 0.5f : 0;
		int i = 0;
		for ( Bar * obj in values )
		{
			if (comparisonChart) {
			if (i%2)
				pointX -= 5;
			else 
				pointX += 5;
			}
				

			// calcualte the bar size
			float value		= [obj value];
			if (value == 0) {
				// outlineWidth = 0;
				// outlineHalfWidth = 0;
			}
			float scale		= ( value / maxValue );
			
			// shift the bar to the top or bottom of the render line
			int pointY	  = lineHeight + ( (lineWidth * 0.5f) * ( ( value >= 0.0f ) ? -1 : 1 ) );
			int barHeight = height * scale;
			CGRect barRect = CGRectMake(pointX, pointY, width, -barHeight);

			// NSLog(@"pointX %i pointY %i barHeight %i", pointX, pointY, barHeight);
			// UIImageView * backImage = [[[UIImageView alloc] initWithFrame:CGRectMake(pointX, pointY-11, 58, 11)] autorelease];
			// UIImage *img    = [UIImage imageNamed:@"chart_blue_bar.png"];
			UIImage *img    = [UIImage imageNamed:[NSString stringWithFormat:@"%@_bar.png", [obj imgColor]]];
			// NSLog(@"%@", [obj imgColor]);
			// [backImage setImage:img];
			// [self addSubview:backImage];
			
			int count = barHeight / 12;
			// NSLog(@"count %i", count);
			// int y = pointY - 11;
			int y; // = pointY-barHeight + 12;
			int x = pointX;
			if (width > 58) x += (width-58)/2;
			int counter = 1;
			int t = 0;
			while (counter <= count) {
				// y = pointY - (11*counter);
				y = pointY - barHeight + (12*counter);
				int h = 12;
				if (h > pointY - y) h = pointY - y;
				// int inY = y;
				// if (inY < pointY-barHeight+11) inY = pointY-barHeight+11;
				// UIImageView *backImage = [[UIImageView alloc] initWithFrame:CGRectMake(pointX, inY, width, 11)];
				// NSLog(@"counter %i y %i h %i img", counter, y, h, [NSString stringWithFormat:@"%@_bar.png", [obj imgColor]]);
				UIImageView *backImage = [[UIImageView alloc] initWithFrame:CGRectMake(x, y, (width>58) ? 58 : width, h)];
				[backImage setImage:img];
				[self addSubview:backImage];
				[backImage release];
				if (t%2) {
					// CGContextSetFillColorWithColor(context,[[obj color] CGColor]);
					// CGContextFillRect(context, CGRectMake(pointX, y, width, h));
				}
				t++;
				counter++;
			}
			// NSLog(@"y: %i", pointY);
			// if (pointY != 0) {
			if (value >= 12 || (maxValue >= 6 && value)) {
			UIImageView *backImage = [[UIImageView alloc] initWithFrame:CGRectMake(x, pointY-barHeight, (width>58) ? 58 : width, 12)];
			// img = [UIImage imageNamed:@"chart_blue_top.png"];
			img    = [UIImage imageNamed:[NSString stringWithFormat:@"%@_top.png", [obj imgColor]]];
			[backImage setImage:img];
			[self addSubview:backImage];
			[backImage release];
			}
			/*
			 int y = pointY - 11;
			while (y > pointY-11-barHeight) {
				NSLog(@"pointX %i pointY %i y %i", pointX, pointY, y);
				[backImage release];
				backImage = [[[UIImageView alloc] initWithFrame:CGRectMake(pointX, y, 58, 11)] autorelease];
				[backImage setImage:img];
				[self addSubview:backImage];
				y -= 11;
			}
			*/
			// draw the under shadow
			/*if ( drawBarShadow && value != 0)
			{
				int offsetX = ( barShadowOffset.x > 0.0f ) ? 1 : -1;
				int additionY = barShadowOffset.y;
				int additionX = barShadowOffset.x - offsetX;
				CGRect barShadow = CGRectMake(pointX + offsetX, pointY, width + additionX, -( barHeight + additionY ) );
				CGContextSetFillColorWithColor(context,[barShadowColor CGColor]);
				CGContextFillRect(context, barShadow);
			}*/
			
			// ------------------------------------------------------------
			// Draw the bar rect
			// ------------------------------------------------------------
			//CGContextSetFillColorWithColor(context,[[obj color] CGColor]);
			//CGContextFillRect(context, barRect);
			// ------------------------------------------------------------
			
			// ------------------------------------------------------------
			// draw the bar highlight.
			// ------------------------------------------------------------
			/*if ( drawBarHighlight )
			{
				float subWidth = ( width * 0.1f > 0 ) ? width * barHighlightPercentage : ( width > 2 ) ?  2 : 1;
				CGRect highlightRect = CGRectMake(pointX, pointY, subWidth, -barHeight );
				
				UIColor * alphaColor = [UIColor colorWithWhite:1 alpha:0.3f];
				CGContextSetFillColorWithColor(context,[alphaColor CGColor]);
				CGContextFillRect(context, highlightRect);
			}*/
			// ------------------------------------------------------------
			
			// ------------------------------------------------------------
			// draw the bar outline
			// ------------------------------------------------------------
			/*if ( drawBarOutline )
			{
				// start the drawing
				CGContextBeginPath(context);
				
				CGContextSetStrokeColorWithColor(context, [barOutlineColor CGColor]);
				
				// set the line width
				CGContextSetLineWidth(context, outlineWidth);
				
				int minX = barRect.origin.x + outlineHalfWidth;
				int minY = barRect.origin.y + ( outlineHalfWidth * ( ( value >= 0.0f ) ? -1 : 1 ) );
				int maxX = barRect.origin.x + barRect.size.width - outlineHalfWidth;
				int maxY = barRect.origin.y + barRect.size.height + ( outlineHalfWidth * ( ( value >= 0.0f ) ? 1 : -1 ) );
				maxY += (value < 0.0f) ? (outlineWidth == 1) ? outlineWidth : 0 : 0;
				maxX += (outlineWidth == 1) ? -outlineWidth : 0;
						 
				CGContextMoveToPoint(context, minX, minY);
				CGContextAddLineToPoint(context, minX, maxY );
				CGContextAddLineToPoint(context, maxX, maxY );
				CGContextAddLineToPoint(context, maxX, minY );
				CGContextAddLineToPoint(context, minX - outlineHalfWidth, minY );
				
				CGContextDrawPath(context, kCGPathStroke);
			}*/
			// ------------------------------------------------------------
			
			// ------------------------------------------------------------
			// draw the percengage value
			// ------------------------------------------------------------
			if ( drawBarPercentages )
			{
				float percengage = 0.0f;
				
				// work out the percentage
				if ( minValue > 0 )
				{
//					percengage = obj.value / maxValue;
				}
				else
				{
//					percengage = obj.value / ( maxValue - minValue );
				}
//				percengage *= 100.0f; // move to be from 0% - 100%
				
				// new percengage
				
				if (!comparisonChart)
					percengage = obj.value * 100 / total;
				else {
					if (i%2)
						percengage = obj.value * 100 / totalB;
					else
						percengage = obj.value * 100 / (total-totalB);
				}
					
				 
				// render
				NSString * str = [NSString stringWithFormat:@"%.1f%@", percengage, @"%"];
				
				if (!showPercengages) {
					// int value = (maxValue * percengage) / 100;
					// NSString * val = (value == 0) ? @" " : [NSString stringWithFormat:@"%i",  [obj value]];
					//str = val;
					// NSLog(@"val: %@", str);
					int value = obj.value;
					str = [NSString stringWithFormat:@"%i", value];
				}


				CGSize size = [str sizeWithFont:fontSmall];
				if ([str isEqualToString:@"0"] || [str isEqualToString:@"0.0%"]) str = @"";
				
				// NSLog(@"value? %f %f %f", maxValue, minValue, percengage);

				NSInteger  x = pointX + ( width * 0.5f ) - ( size.width * 0.5f );				
				NSInteger  y  = pointY - ( ( barHeight * 0.5f ) - ( ( barHeight < 0 ) ? [font pointSize] * 0.5f : 0 ) );
				if (comparisonChart) x += 2.5;

				NSInteger ymax = y + ( ( barHeight < 0 ) ? 0 : -[fontSmall pointSize] );
				
				// is the percentage in a good place
				if ( !CGRectContainsPoint( barRect, CGPointMake( x, ymax) ) )
				{
					// if the point above the bar
					y = pointY - barHeight + ( ( barHeight < 0 ) ? [fontSmall pointSize] : -4 );
				}
				if (!showValueInside)
					y = pointY - 1 - barHeight + ( ( barHeight < 0 ) ? [fontSmall pointSize] : -4 );
				
				CGContextSetAllowsAntialiasing(context, YES);
				
				CGAffineTransform textTransform = CGAffineTransformMake( 1.0, 0.0f, 0.0f, -1.0f, 0.0f, 0.0f );
				CGContextSetTextMatrix(context, textTransform);
				
				// Set the Context Font the custom one				
				CGContextSelectFont(context, [fontSmall.fontName UTF8String], fontSmall.pointSize, kCGEncodingMacRoman);
				
				//The following code is for Drawing Text
				CGContextSetTextDrawingMode (context, kCGTextFill); 
				
				// set the font color
				CGContextSetFillColorWithColor(context, [fontColor CGColor]);
				
				// draw the text.
				CGContextShowTextAtPoint(context, x, y, [str UTF8String], str.length);
				// CGContextShowTextAtPoint(context, x, y, [val UTF8String], val.length);

				CGContextSetAllowsAntialiasing(context, NO);
			}
			// ------------------------------------------------------------
						
// bh
			// if (comparisonChart) {
			//if (i%2)
			//	pointX += 5;
			//else 
			//	pointX -= 5;
			i++;
			
			// move the point on to the start of the next bar
			pointX += width + seperator;
		}
		
		CGContextSetAllowsAntialiasing(context, YES);
	}
	// ----------------------------------------
	
	// ----------------------------------------
	// draw the background area
	// ----------------------------------------
	if ( drawLine )
	{
		// Work out the edge of the graph
		CGPoint verticalStart	 = area.origin;
		CGPoint verticalEnd		 = CGPointMake( verticalStart.x, verticalStart.y + area.size.height );
		
		// Work out the line of the
		CGPoint horazontalStart  = CGPointMake( area.origin.x, lineHeight );
		CGPoint horazontalEnd    = CGPointMake( area.origin.x + area.size.width, lineHeight );
		
		// start the drawing
		CGContextBeginPath(context);
		
		// set the color of the stroke
		CGContextSetStrokeColorWithColor(context, [lineColor CGColor]);
		
		// set the line width
		CGContextSetLineWidth(context, lineWidth);
		// turn off antialasing
		CGContextSetAllowsAntialiasing(context, NO);
		
		// draw the vertical line
		CGContextMoveToPoint(context, verticalStart.x, verticalStart.y-1);
		CGContextAddLineToPoint(context, verticalEnd.x, verticalEnd.y );
		
		// draw the horazontal line
		CGContextMoveToPoint(context, horazontalStart.x, horazontalStart.y);
		CGContextAddLineToPoint(context, horazontalEnd.x, horazontalEnd.y );
		
		CGContextDrawPath(context, kCGPathStroke);

		// --------------------------------------------
		// draw a short line down to seperate the bars
		// --------------------------------------------
		if ( drawBarNames && [values count] )
		{
			int size	   = [font pointSize];
			int pointX     = area.origin.x + ( lineWidth / 2 );
			int pointY	   = area.origin.y + area.size.height;
			int shift      = ( [values count] ) ? ( area.size.width - ( lineWidth / 2 )) / [values count] : 0;
			
			if (comparisonChart) pointX += 5;
			
			CGContextSetStrokeColorWithColor(context, [barOutlineColor CGColor]);
			CGContextSetLineWidth(context, 1);
			
			for ( int i=0; i<[values count] - 1; i++ )
			{
				pointX += shift;
				
				CGContextBeginPath(context);
				CGContextMoveToPoint(context, pointX, pointY );
				CGContextAddLineToPoint(context, pointX, pointY + ( size * 0.5f ) );
				CGContextDrawPath(context, kCGPathStroke);			
			}
		}
		// --------------------------------------------
		
		CGContextSetAllowsAntialiasing(context, YES);
	}
	// ----------------------------------------
	
}

#pragma mark ---------------------------------------------------------
#pragma mark === End View Functions  ===
#pragma mark ---------------------------------------------------------

#pragma mark ---------------------------------------------------------
#pragma mark === Public Functions  ===
#pragma mark ---------------------------------------------------------

// -------------------------------------------------------------------
// remove all the current bar objects
// -------------------------------------------------------------------
- (void) removeAllBars
{
	[values removeAllObjects];
}

// -------------------------------------------------------------------
// Function : Remove the bar at the index provided
// -------------------------------------------------------------------
- (void) removeBarAtIndex:(NSInteger)_index
{
	[values removeObjectAtIndex:_index];
}

// -------------------------------------------------------------------
// Function : Remove the first bar found with the name provided
// -------------------------------------------------------------------
- (void) removeBarWithName:(NSString*)_name
{
	Bar * bar = nil;
	for ( Bar * obj in values )
	{
		if ( [obj.name compare:_name] == NSOrderedSame )
		{
			bar = obj;
			break;
		}
	}
	
	[self removeBar:bar];
}

// -------------------------------------------------------------------
// Function : Remove the bar object provided
// -------------------------------------------------------------------
- (void) removeBar:(Bar*)_bar
{
	[values removeObject:_bar];
}

// -------------------------------------------------------------------
// Function : Add a bar to graph
// -------------------------------------------------------------------
- (Bar*) addBarWithName:(NSString*)_name WithColor:(UIColor*)_color  WithImgColor:(NSString*)_imgColor AndValue:(float)_value
{
	Bar * bar = [[Bar alloc] initWithName:_name  AndColor:_color AndImgColor:_imgColor AndValue:_value];
	[values addObject:bar];
	[bar release];
	return bar;
}

- (Bar*) addBarWithName:(NSString*)_name WithColor:(UIColor*)_color AndValue:(float)_value
{
	Bar * bar = [[Bar alloc] initWithName:_name  AndColor:_color AndImgColor:@"" AndValue:_value];
	[values addObject:bar];
	[bar release];
	return bar;
}

// -------------------------------------------------------------------
// Function : return the bar at the index passed in
// -------------------------------------------------------------------
- (Bar*) barAtIndex:(NSInteger)_index
{
	return [values objectAtIndex:_index];
}

// -------------------------------------------------------------------
// Function : return the first bar with the name that was passed in
// ------------------------------------------------------------------- 
- (Bar*) barWithName:(NSString*)_name
{
	for ( Bar * obj in values )
	{
		if ( [obj.name compare:_name] == NSOrderedSame )
		{
			return obj;
		}
	}

	return nil;
}

#pragma mark ---------------------------------------------------------
#pragma mark === End Public Functions  ===
#pragma mark ---------------------------------------------------------

@end
