// ===
// Functions test
// ===

#import "Functions.h"
#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import <SystemConfiguration/SCNetworkReachability.h>
#include <netinet/in.h>
#include "Config.h"
#include "app_builderAppDelegate.h"
#import "FontItem.h"
#import "TBXML.h"

@implementation Functions

// http://iphonedevelopertips.com/general/using-nsscanner-to-convert-hex-to-rgb-color.html
+ (UIColor *) colorForHex:(NSString *)hexColor {
	hexColor = [[hexColor stringByTrimmingCharactersInSet:
				 [NSCharacterSet whitespaceAndNewlineCharacterSet]
				 ] uppercaseString];  
	
    // String should be 6 or 7 characters if it includes '#'  
    if ([hexColor length] < 6) 
		return [UIColor blackColor];  
	
    // strip # if it appears  
    if ([hexColor hasPrefix:@"#"]) 
		hexColor = [hexColor substringFromIndex:1];  
	
    // if the value isn't 6 characters at this point return 
    // the color black	
    if ([hexColor length] != 6) 
		return [UIColor blackColor];  
	
    // Separate into r, g, b substrings  
    NSRange range;  
    range.location = 0;  
    range.length = 2; 
	
    NSString *rString = [hexColor substringWithRange:range];  
	
    range.location = 2;  
    NSString *gString = [hexColor substringWithRange:range];  
	
    range.location = 4;  
    NSString *bString = [hexColor substringWithRange:range];  
	
    // Scan values  
    unsigned int r, g, b;  
    [[NSScanner scannerWithString:rString] scanHexInt:&r];  
    [[NSScanner scannerWithString:gString] scanHexInt:&g];  
    [[NSScanner scannerWithString:bString] scanHexInt:&b];  
	
    return [UIColor colorWithRed:((float) r / 255.0f)  
                           green:((float) g / 255.0f)  
                            blue:((float) b / 255.0f)  
                           alpha:1.0f];  
	
}

+(TBXMLElement *)readConfigXmlFile {
	NSString * databaseName = @"app_builder.xml";
	NSArray *documentPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDir = [documentPaths objectAtIndex:0];
	NSString *databasePath = [documentsDir stringByAppendingPathComponent:databaseName];
	// NSLog(@"Path: %@", databasePath);
	NSData *myXMLData = [NSData dataWithContentsOfFile:databasePath];
	TBXML * tbxml = [TBXML tbxmlWithXMLData:myXMLData];
	TBXMLElement * root = tbxml.rootXMLElement;
	return root;
}

+(UIImage *)resizeImage:(UIImage *)img width:(CGFloat)targetWidth height:(CGFloat)targetHeight {
	CGSize imageSize = img.size;
	CGFloat width = imageSize.width;
	CGFloat height = imageSize.height;
	CGSize targetSize = {.width = targetWidth, .height = targetHeight};
	// CGSize targetSize = {.width = 160, .height = 200};
	// CGFloat targetWidth = targetSize.width;
	// CGFloat targetHeight = targetSize.height;
	CGFloat scaleFactor = 0.0;
	CGFloat scaledWidth = targetWidth;
	CGFloat scaledHeight = targetHeight;
	CGPoint thumbnailPoint = CGPointMake(0.0,0.0);	
	
	if (CGSizeEqualToSize(imageSize, targetSize) == NO) 
	{
        CGFloat widthFactor = targetWidth / width;
        CGFloat heightFactor = targetHeight / height;
		
        if (widthFactor > heightFactor) 
			scaleFactor = widthFactor; // scale to fit height
        else
			scaleFactor = heightFactor; // scale to fit width
        scaledWidth  = width * scaleFactor;
        scaledHeight = height * scaleFactor;
		
        // center the image
        if (widthFactor > heightFactor)
		{
			thumbnailPoint.y = (targetHeight - scaledHeight) * 0.5; 
		}
        else 
			if (widthFactor < heightFactor)
			{
				thumbnailPoint.x = (targetWidth - scaledWidth) * 0.5;
			}
	}       
	
	UIGraphicsBeginImageContext(targetSize); // this will crop
	
	CGRect thumbnailRect = CGRectZero;
	thumbnailRect.origin = thumbnailPoint;
	thumbnailRect.size.width  = scaledWidth;
	thumbnailRect.size.height = scaledHeight;
	
	[img drawInRect:thumbnailRect];
	
	img = UIGraphicsGetImageFromCurrentImageContext();
	if(img == nil) 
        NSLog(@"could not scale image");
	
	//pop the context to get back to the default
	UIGraphicsEndImageContext();
	
	return img;
}



/*
 - (BOOL) connectedToNetwork
{
    // Create zero addy
    struct sockaddr_in zeroAddress;
    bzero(&zeroAddress, sizeof(zeroAddress));
    zeroAddress.sin_len = sizeof(zeroAddress);
    zeroAddress.sin_family = AF_INET;
    
    // Recover reachability flags
    SCNetworkReachabilityRef defaultRouteReachability = SCNetworkReachabilityCreateWithAddress(NULL, (struct sockaddr *)&zeroAddress);
    SCNetworkReachabilityFlags flags;
    
    BOOL didRetrieveFlags = SCNetworkReachabilityGetFlags(defaultRouteReachability, &flags);
    CFRelease(defaultRouteReachability);
    
    if (!didRetrieveFlags)
    {
        printf("Error. Could not recover network reachability flags\n");
        return NO;
    }
    
    BOOL isReachable = ((flags & kSCNetworkFlagsReachable) != 0);
    BOOL needsConnection = ((flags & kSCNetworkFlagsConnectionRequired) != 0);
	BOOL nonWiFi = flags & kSCNetworkReachabilityFlagsTransientConnection;
	
	// return ((isReachable && !needsConnection) || nonWiFi) ? YES : NO;
    // return (isReachable && !needsConnection) ? YES : NO;
	return ((isReachable && !needsConnection) || nonWiFi) ? 
	(([[NSURLConnection alloc] initWithRequest:[NSURLRequest 
												requestWithURL: [NSURL URLWithString:@"http://www.apple.com/"] 
												cachePolicy:NSURLRequestReloadIgnoringLocalCacheData timeoutInterval:20.0] 
									  delegate:self]) ? YES : NO) : NO;
}
 */

//+(void)updateConfig:(NSString *)configName withValue:(NSString *)newValue {
//	Config *cnf;
//	NSString *value;
//	cnf = [[Config alloc] initWithName:configName];
//	value = cnf.value;
//	if([value isEqualToString:@""] == TRUE) {
//		NSInteger primaryKey = [Config insertValue:configName value:newValue];
//		NSLog(@"v: %d", primaryKey);
//	} else {
//		[Config updateValue:configName value:newValue];
//	}
//}
//
//+(NSString *) loadConfig:(NSString *)configName {
//	Config *cnf;
//	NSString *value;
//	cnf = [[Config alloc] initWithName:configName];
//	value = cnf.value;
//	return value;
//}

+ (NSString *)fontName:(NSString *)font fontStyle:style {
	NSString *actualFont;
	if ([style isEqualToString:@"bold"]) {
		actualFont = [font stringByAppendingString:@"-Bold"];
	} else if ([style isEqualToString:@"italic"]) {
		if ([font isEqualToString:@"Arial"])
			actualFont = [font stringByAppendingString:@"-Italic"];
		else
			actualFont = [font stringByAppendingString:@"-Oblique"];
	} else if ([style isEqualToString:@"bold-italic"]) {
		if ([font isEqualToString:@"Arial"])
			actualFont = [font stringByAppendingString:@"-BoldItalic"];
		else
			actualFont = [font stringByAppendingString:@"-BoldOblique"];
	} else {
		actualFont = font;
	}
	if ([font isEqualToString:@"Arial"])
		actualFont = [actualFont stringByAppendingString:@"MT"];
	return actualFont;
}

+ (void) initToolBar:(UIView *)actualView withTitle:(NSString*) title
{
	[self addToolBar:actualView withTitle:title actualId:@"0"];	
}

+ (void) addToolBar:(UIView *)actualView withTitle:(NSString*) title
actualId: (NSString *)actualId {

    CGRect mainViewBounds = [[UIScreen mainScreen] applicationFrame];
    
	NSArray *documentPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDir = [documentPaths objectAtIndex:0];
	NSString *filePath = [documentsDir stringByAppendingPathComponent:@"/gradientBackground.png"];
	NSData *imgData = [NSData dataWithContentsOfFile:filePath];
	UIImage *tempImg = [[UIImage alloc] initWithData:imgData];
	UIImageView *bgImageView = [[UIImageView alloc] initWithImage:tempImg];
	// [bgImageView setImage:tempImg];
	bgImageView.frame = CGRectMake(0, 44, 320, CGRectGetHeight(mainViewBounds));
	// bgImageView.frame = CGRectMake(0, 44, actualView.frame.size.width, actualView.frame.size.height);
	// if (actualView.tag != 29876)
		bgImageView.frame = CGRectMake(0, 0, actualView.frame.size.width, CGRectGetHeight(mainViewBounds));
	// bgImageView.frame = CGRectMake(0, 44, CGRectGetWidth(mainViewBounds), CGRectGetHeight(mainViewBounds));
	bgImageView.autoresizingMask = UIViewAutoresizingFlexibleWidth | UIViewAutoresizingFlexibleHeight;
	bgImageView.autoresizingMask = (UIViewAutoresizingFlexibleBottomMargin 
								| UIViewAutoresizingFlexibleRightMargin 
								| UIViewAutoresizingFlexibleLeftMargin 
								| UIViewAutoresizingFlexibleTopMargin 
								| UIViewAutoresizingFlexibleWidth
									| UIViewAutoresizingFlexibleHeight);
	[actualView addSubview:bgImageView];
    [actualView sendSubviewToBack:bgImageView];
	[bgImageView release];
	[tempImg release];

	BOOL useAd = FALSE;
	NSString * imgUrl = @""; // [[NSString alloc] init];
	// NSString *title =@"Video";
	UIToolbar *toolbar = [UIToolbar new];
	toolbar.tag = 10001;
	[toolbar sizeToFit];
	CGFloat toolbarHeight = [toolbar frame].size.height;
	// CGRect mainViewBounds = [[UIScreen mainScreen] applicationFrame];
	// [toolbar setFrame:CGRectMake(CGRectGetMinX(mainViewBounds), CGRectGetMinY(mainViewBounds) + CGRectGetHeight(mainViewBounds) - 108.0, CGRectGetWidth(mainViewBounds), toolbarHeight)];
	[toolbar setFrame:CGRectMake(0, 0, CGRectGetWidth(mainViewBounds), toolbarHeight)];
	[toolbar setFrame:CGRectMake(0, 0, actualView.frame.size.width, toolbarHeight)];
    NSString * colorStyle = [self headerStyle];
    if (![colorStyle isEqualToString:@""])
    {
        toolbar.barStyle = UIBarStyleBlackTranslucent;
        toolbar.tintColor = [self colorForHex:colorStyle];
        toolbar.backgroundColor = [self colorForHex:colorStyle];
    }
	[actualView addSubview:toolbar];
	
	// NSArray *documentPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	// NSString *documentsDir = [documentPaths objectAtIndex:0];
	NSString *fileName = [documentsDir stringByAppendingPathComponent:@"ads.xml"];
	NSData * adsXml = [NSData dataWithContentsOfFile:fileName];
	TBXML * tbxml = [TBXML tbxmlWithXMLData:adsXml];
	TBXMLElement * rootXMLElement = tbxml.rootXMLElement;
	TBXMLElement * childXMLElement;
	if (rootXMLElement) {
		// childXMLElement = rootXMLElement->firstChild;
		childXMLElement = [TBXML childElementNamed:@"modules_advertising" parentElement:rootXMLElement];
		// NSLog(@"? %@", [TBXML elementName:childXMLElement]);
		childXMLElement = childXMLElement->firstChild;
		BOOL f = FALSE;
		do {
			NSString * idView = [TBXML valueOfAttributeNamed:@"id" forElement:childXMLElement];
			NSLog(@"..? %@ -? %@", idView, actualId);
			if ([actualId isEqualToString:idView]) {
				TBXMLElement * adsXMLElement = childXMLElement->firstChild;
				do {
					NSString * adId = [TBXML textForElement:adsXMLElement];
					BOOL g = FALSE;
					// NSLog(@"**? %@", adId);
					TBXMLElement * adXMLElement =  [TBXML childElementNamed:@"ads_settings" parentElement:rootXMLElement];
					adXMLElement = adXMLElement->firstChild;
					do {
						if ([adId isEqualToString:[TBXML valueOfAttributeNamed:@"id" forElement:adXMLElement]]) {
							// NSLog(@"....");
							g = TRUE;
							TBXMLElement * typeXMLElement =  [TBXML childElementNamed:@"ads_type" parentElement:adXMLElement];
							if ([@"S" isEqualToString:[TBXML textForElement:typeXMLElement]]) {
								useAd = TRUE;
								TBXMLElement * imgXMLElement =  [TBXML childElementNamed:@"ads_resource_image" parentElement:adXMLElement];
								imgUrl = [TBXML textForElement:imgXMLElement];
								NSLog(@"img %@" , imgUrl);
							}
						}
					} while ((adXMLElement = adXMLElement->nextSibling) && !g);
					
				} while ((adsXMLElement = adsXMLElement->nextSibling));
				f = TRUE;
			}
		} while ((childXMLElement = childXMLElement->nextSibling) && !f);
	}
	toolbar.autoresizingMask = (UIViewAutoresizingFlexibleWidth);
	toolbar.autoresizingMask = (UIViewAutoresizingFlexibleBottomMargin 
							  | UIViewAutoresizingFlexibleRightMargin 
							  | UIViewAutoresizingFlexibleLeftMargin 
	/*						  | UIViewAutoresizingFlexibleTopMargin */
							  | UIViewAutoresizingFlexibleWidth);

	UILabel *label = [[UILabel alloc] initWithFrame:
                      CGRectMake(0,8,actualView.frame.size.width,30)];
	label.tag = 10002;
	label.autoresizingMask = (UIViewAutoresizingFlexibleBottomMargin 
							   | UIViewAutoresizingFlexibleRightMargin 
							   | UIViewAutoresizingFlexibleLeftMargin 
							   | UIViewAutoresizingFlexibleTopMargin 
							   | UIViewAutoresizingFlexibleWidth);
    // label.autoresizingMask = UIViewAutoresizingFlexibleWidth;
	// [label setFrame:CGRectMake(0, 8, CGRectGetWidth(mainViewBounds), 30)];
    label.text = title; // @"Project";
    label.backgroundColor = [UIColor clearColor];
    // label.backgroundColor = [UIColor blueColor];
    label.textColor = [UIColor whiteColor];
    label.font = [UIFont boldSystemFontOfSize:16];
    label.textAlignment = UITextAlignmentCenter;
	[toolbar addSubview:label];
    [label release];

	if (useAd) {
//		NSArray *paths = NSSearchPathForDirectoriesInDomains(
//															 NSDocumentDirectory, NSUserDomainMask, YES);
//		NSString *docDirectory = [paths objectAtIndex:0];
//		docDirectory = [docDirectory stringByAppendingPathComponent:@"/logo.png"];
//		NSData *imgData = [[NSData alloc] initWithContentsOfFile:docDirectory];
		NSData *imgData = [[NSData alloc] initWithContentsOfURL:[NSURL URLWithString:imgUrl]];
		UIImage *img    = [[UIImage alloc] initWithData:imgData];
		CGRect myImageRect = CGRectMake(0, 0, 320, 44);
		UIImageView *logoImage = [[UIImageView alloc] initWithFrame:myImageRect];
		[logoImage setImage:img];
		[toolbar addSubview:logoImage];
        [img release];
        [imgData release];
		[logoImage release];
	}
    [toolbar release];

}

+ (void) addStatusBar:(UIView *)actualView withTabBar:(BOOL)useTabBarSpace
{
	
	int tabSpace = 0;
	if (useTabBarSpace)
		tabSpace = 48;
	
	UIToolbar * toolbar = [UIToolbar new];
	[toolbar sizeToFit];
	toolbar.tag = 11003;
	CGFloat toolbarHeight = 34; // [toolbar frame].size.height;
	CGRect mainViewBounds = [[UIScreen mainScreen] applicationFrame];
	// [toolbar setFrame:CGRectMake(CGRectGetMinX(mainViewBounds), CGRectGetMinY(mainViewBounds) + CGRectGetHeight(mainViewBounds) - 108.0, CGRectGetWidth(mainViewBounds), toolbarHeight)];
	[toolbar setFrame:CGRectMake(0, 426-tabSpace, CGRectGetWidth(mainViewBounds), toolbarHeight)];
	[toolbar setFrame:CGRectMake(0, CGRectGetHeight(mainViewBounds)-tabSpace-toolbarHeight, CGRectGetWidth(mainViewBounds), toolbarHeight)];
	[toolbar setFrame:CGRectMake(0, actualView.frame.size.height-toolbarHeight, actualView.frame.size.width, toolbarHeight)];
	toolbar.autoresizingMask = (UIViewAutoresizingFlexibleBottomMargin 
								| UIViewAutoresizingFlexibleRightMargin 
								| UIViewAutoresizingFlexibleLeftMargin 
								| UIViewAutoresizingFlexibleTopMargin 
								| UIViewAutoresizingFlexibleWidth);
	
	// toolbar.barStyle = UIBarStyleBlackTranslucent;
	// toolbar.tintColor = [UIColor blueColor];
	// toolbar.alpha = 0.7;

    NSString * colorStyle = [self headerStyle];
    if (![colorStyle isEqualToString:@""])
    {
        toolbar.barStyle = UIBarStyleBlackTranslucent;
        toolbar.tintColor = [self colorForHex:colorStyle];
        toolbar.backgroundColor = [self colorForHex:colorStyle];
    }
	[actualView addSubview:toolbar];
	[toolbar release];
}	

# pragma mark -

// + (IBAction)goMenu:(id) sender{
+ (void)goMenu:(id) sender{
	// -(void)goMenu{
	Config * cnf = [[Config alloc] init];
	// NSString * actualView = [cnf loadValue:@"displayView"];
	NSString * displayView = [cnf loadValue:@"displayView"];
	[cnf updateValue:@"goUrl" value:@"---"];
	// cnf = [[Config alloc] initWithName:@"displayId"];
	// NSString * displayId = cnf.value;
	app_builderAppDelegate *appDelegate = [[UIApplication sharedApplication] delegate];
	NSString * actualView = [cnf loadValue:@"actualView"];
	NSLog(@"Menu: %@ : %@", displayView, actualView);
	if (![displayView isEqualToString:@"1"]) {
		[cnf updateValue:@"displayView" value:@"1"];
		[cnf updateValue:@"displayId" value:@"0"];
		if ([actualView isEqualToString: displayView]) {
			actualView = @"1";
			NSLog(@"Menu -: %@", actualView);
			[appDelegate displayView:actualView];
		} else {
			NSLog(@"Menu *: %@", actualView);
			[appDelegate displayView:actualView];
		}
	} else {
		NSLog(@"Menu --- : %@", displayView);
		[appDelegate displayView:@"1"];
	}
	
	[cnf release];
}

/*+ (IBAction)goView:(id) sender{
	// app_builderAppDelegate *appDelegate = [[UIApplication sharedApplication] delegate];
	// [appDelegate goDetail];
}*/

// http://blog.sallarp.com/iphone-uilabel-multiline-dynamic-height/
+(float) calculateHeightOfTextFromWidth:(NSString*) text: (UIFont*)withFont: (float)width :(UILineBreakMode)lineBreakMode
{
	[text retain];
	[withFont retain];
	CGSize suggestedSize = [text sizeWithFont:withFont constrainedToSize:CGSizeMake(width, FLT_MAX) lineBreakMode:lineBreakMode];
	
	[text release];
	[withFont release];
	
	return suggestedSize.height;
}


+(NSString *) loadFromTabBar:(NSString *)actualTitle {
	Config * cnf;
	cnf = [[Config alloc] init];
	NSString * value = [cnf loadValue:@"actualId"];
	NSString *actualId = @"";
	
	TBXMLElement * root = [Functions readConfigXmlFile];
	TBXMLElement * childXMLElement;
	TBXMLElement * feedsXMLElement;
	TBXMLElement * appXMLElement;
	if (root) {
		// if (root->firstChild) 
		//	element = root->firstChild;
		
		BOOL f = FALSE;
		childXMLElement = root->firstChild;
		do {
			if ([[TBXML elementName:childXMLElement] isEqualToString:@"module"]) {
				if ([[TBXML valueOfAttributeNamed:@"id" forElement:childXMLElement] 
					 isEqualToString:value]) {
						f = TRUE;
						feedsXMLElement = childXMLElement;
				}
			}
		} while ((childXMLElement = childXMLElement->nextSibling) && (f == FALSE));
		
		if (f) {
			appXMLElement =  feedsXMLElement->firstChild;
			// appXMLElement =  appXMLElement->firstChild;
			// NSString *title;
			// NSString *icon;

			
			do {
				if ([[TBXML elementName:appXMLElement] isEqualToString:@"field_set"]) {
					if ([[TBXML valueOfAttributeNamed:@"field_name" forElement:appXMLElement] isEqualToString:@"tab_bar"]) {
						childXMLElement = appXMLElement->firstChild;;
						do {
							NSString *attribItem = [TBXML valueOfAttributeNamed:@"name" forElement:childXMLElement];
							// NSString *attribType = [TBXML valueOfAttributeNamed:@"type" forElement:childXMLElement];
							NSString *attribValue = [TBXML valueOfAttributeNamed:@"value" forElement:childXMLElement];
							if ([attribItem isEqualToString:@"tap_item"] && 
								[attribValue isEqualToString:actualTitle]) {
								actualId = [TBXML valueOfAttributeNamed:@"link_to_ref" forElement:childXMLElement];
							}
						} while ((childXMLElement = childXMLElement->nextSibling));
						
					}
				}
			} while ((appXMLElement = appXMLElement->nextSibling));
		}
	}
	// NSLog(@"ai: %@", actualId);
	
	[cnf release];
	
	return actualId;
}

+ (NSString *) headerStyle
{
	TBXMLElement * root = [Functions readConfigXmlFile]; // tbxml.rootXMLElement;
    NSString * style = @"";
	if (root) {
		TBXMLElement * appXMLElement = [TBXML childElementNamed:@"app_settings" parentElement:root];
		appXMLElement = [TBXML childElementNamed:@"styles" parentElement:appXMLElement];
        style = [TBXML valueOfAttributeNamed:@"color_background_title" forElement:appXMLElement];
	}
    return style;
}

+(FontItem *) loadFont:(NSString *)idFont {
	TBXMLElement * root = [Functions readConfigXmlFile]; // tbxml.rootXMLElement;
	FontItem *font = [[[FontItem alloc] init] autorelease];
	if (root) {
		TBXMLElement * appXMLElement = [TBXML childElementNamed:@"app_settings" parentElement:root];
		appXMLElement = [TBXML childElementNamed:@"styles" parentElement:appXMLElement];
		BOOL f = FALSE;
		TBXMLElement * fontXMLElement = appXMLElement->firstChild;
		do {
			if ([[TBXML valueOfAttributeNamed:@"style_id" forElement:fontXMLElement] isEqualToString:idFont]) {
				f = TRUE;
				// NSLog(@"f: %@", [TBXML valueOfAttributeNamed:@"name" forElement:fontXMLElement]);
				// font = [FontItem initWithName:[TBXML valueOfAttributeNamed:@"name" forElement:fontXMLElement]];
				font.nameFont = [TBXML valueOfAttributeNamed:@"name" forElement:fontXMLElement];
				font.sizeFont = [TBXML valueOfAttributeNamed:@"size" forElement:fontXMLElement];
				font.styleFont = [TBXML valueOfAttributeNamed:@"style" forElement:fontXMLElement];
				font.colorFont = [TBXML valueOfAttributeNamed:@"color" forElement:fontXMLElement];
			}
		} while ((fontXMLElement = fontXMLElement->nextSibling) && (f == FALSE));
	}
	return font;
}

+ (void) sendWebtrends:(NSString *)currPath eventDescr:(NSString *)eventDescr 
			 eventType:(NSString *)eventType 
		  contentGroup:(NSString *)contentGroup 
			 mediaName:(NSString *)mediaName 
			 mediaType:(NSString *)mediaType 
			mediaEvent:(NSString *)mediaEvent
{
	
	/*
	app_builderAppDelegate *appDelegate = [[UIApplication sharedApplication] delegate];
	NSString *eventPath = [NSString stringWithFormat:@"%@/%@", [appDelegate appPath], currPath];
	NSLog(@"pt: %@", eventPath);
	WTEvent *myEvent;
	
	 if (mediaName == nil || ([mediaName length] == 0))
		myEvent = [WTEvent eventForContentView: eventPath
									eventDescr: eventDescr 
									 eventType: eventType
								  contentGroup: contentGroup ];
	else
		myEvent = [WTEvent eventForMediaView: eventPath 
								  eventDescr: eventDescr
								   eventType: eventType 
								contentGroup: contentGroup 
								   mediaName: mediaName 
								   mediaType: mediaType 
								  mediaEvent: mediaEvent ];
	/ *
	 [myEvent setValue:@"some val" forParameter:WTParameter];
	 [myEvent setValue:@"some val" forParameter:WTParameterMediaEvent];
	 [myEvent setValue:@"some val" forParameter:WTParameterMediaName];
	 [myEvent setValue:@"some val" forParameter:WTParameterMediaType];
	 [myEvent setValue:@"custom_key" forCustomParameter:@"custom_value"];
	 * /
	[self trackEvent:myEvent];
	*/
	
}

+ (void) useAd:(NSString *)moduleId {
	// NSString * databaseName = @"ad_central.xml";
	NSString * databaseName = @"ads.xml";
	NSArray *documentPaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDir = [documentPaths objectAtIndex:0];
	NSString *databasePath = [documentsDir stringByAppendingPathComponent:databaseName];
	// NSLog(@"Path: %@", databasePath);
	NSData *myXMLData = [[NSData alloc] initWithContentsOfFile:databasePath];
	TBXML * tbxml = [[TBXML tbxmlWithXMLData:myXMLData] retain];
	TBXMLElement * rootXMLElement = tbxml.rootXMLElement;
    [tbxml release];
    [myXMLData release];
 
 	BOOL f = FALSE;
	TBXMLElement *adXMLElement;
	TBXMLElement * childXMLElement; // = [TBXML childElementNamed:@"modules_advertising" parentElement:rootXMLElement];
	childXMLElement = rootXMLElement->firstChild;
	// childXMLElement = childXMLElement->firstChild;
	if (!(childXMLElement == nil)) {
		// NSLog(@"not null");
		
		do {
			// NSLog(@"mod_ad? %@", [TBXML elementName:childXMLElement] );
			if ([[TBXML elementName:childXMLElement] isEqualToString:@"modules_advertising"]) {
				TBXMLElement * adsXMLElement = childXMLElement->firstChild;
				do {
					if ([[TBXML elementName:adsXMLElement] isEqualToString:@"module"]) {
						NSString *idItem = [TBXML valueOfAttributeNamed:@"id" forElement:adsXMLElement];
						// NSString *type = [TBXML valueOfAttributeNamed:@"type" forElement:childXMLElement];
						// NSLog(@"moduleId : %@ | idItem: %@ ", moduleId, idItem);
						if ([moduleId isEqualToString:idItem]) {
							f = TRUE;
							adXMLElement = adsXMLElement;
						}
					}
				} while ((adsXMLElement = adsXMLElement->nextSibling) && f == FALSE);
				
			}
		} while ((childXMLElement = childXMLElement->nextSibling) && (f == FALSE));
		
	}
	
	if (f) {
		NSMutableArray *ads = [[NSMutableArray alloc] init];
		TBXMLElement * adsXMLElement = adXMLElement->firstChild;
		do {
			NSString * adId = [TBXML textForElement:adsXMLElement];
			// NSLog(@"adId: %@", adId);
			f = FALSE;
			childXMLElement = [TBXML childElementNamed:@"ads_settings" parentElement:rootXMLElement];
			childXMLElement = childXMLElement->firstChild;
			do {
				// NSLog(@"idItem: %@", [TBXML elementName:childXMLElement]);
				NSString *idItem = [TBXML valueOfAttributeNamed:@"id" forElement:childXMLElement];
				// NSLog(@"idItem: %@", idItem);
				if ([adId isEqualToString:idItem]) {
					// TBXMLElement * valueXMLElement = [TBXML childElementNamed:@"ads_name" parentElement:childXMLElement];
					// NSString * name = [TBXML textForElement:valueXMLElement];
					// valueXMLElement = [TBXML childElementNamed:@"ads_link" parentElement:childXMLElement];
					// NSString * link = [TBXML textForElement:valueXMLElement];
					TBXMLElement * valueXMLElement = [TBXML childElementNamed:@"ads_start_date" parentElement:childXMLElement];
					NSString * start = [TBXML textForElement:valueXMLElement];
					valueXMLElement = [TBXML childElementNamed:@"ads_end_date" parentElement:childXMLElement];
					NSString * end = [TBXML textForElement:valueXMLElement];
					// valueXMLElement = [TBXML childElementNamed:@"ads_resource_image" parentElement:childXMLElement];
					// NSString * image = [TBXML textForElement:valueXMLElement];
					valueXMLElement = [TBXML childElementNamed:@"ads_type" parentElement:childXMLElement];
					NSString * type = [TBXML textForElement:valueXMLElement];
					if ([type isEqualToString:@"A"]) {
						valueXMLElement = [TBXML childElementNamed:@"ads_duration" parentElement:childXMLElement];
						NSString * duration = [TBXML textForElement:valueXMLElement];
						valueXMLElement = [TBXML childElementNamed:@"ads_interval" parentElement:childXMLElement];
						NSString * interval = [TBXML textForElement:valueXMLElement];
						NSDate *today = [NSDate date];
						NSDateFormatter *dateformatter = [[NSDateFormatter alloc] init];
						[dateformatter setDateFormat:@"MM/dd/yyyy"]; // hh:mm:ss"];
						// NSString *launchDay = [[NSString alloc] initWithString:start];
						NSDate *currentDate = [dateformatter dateFromString:start];
						// NSString *endDay = [[NSString alloc] initWithString:end];
						NSDate *endDate = [dateformatter dateFromString:end];
						// NSLog(@"start: %@ %f", start, [today timeIntervalSinceDate:currentDate]);
						// NSLog(@"end: %@ %f", end, [today timeIntervalSinceDate:endDate]);
                        [dateformatter release];
						if ([today timeIntervalSinceDate:currentDate] >= 0 && 
							[today timeIntervalSinceDate:endDate] <= 0 ) {
							// NSLog(@"name %@", name);
							NSArray * ad = [NSArray arrayWithObjects: adId, duration, interval, nil];
							[ads addObject:ad];
						}
						
					}
					// NSLog(@"a: %@", name);
				}
			} while ((childXMLElement = childXMLElement->nextSibling) && (f == FALSE));
		} while ((adsXMLElement = adsXMLElement->nextSibling));
		if ([ads count]) {
			// int adSel = (random() % [ads count]);
			NSArray * ad = [ads objectAtIndex: (random() % [ads count])];
			NSString * idAd = [ad objectAtIndex:0];
			NSString * duration = [ad objectAtIndex:1];
			NSString * interval = [ad objectAtIndex:2];
			
			Config *cnf = [[Config alloc] init];
			
			[cnf updateValue:@"viewAd" value:@"1"];
			[cnf updateValue:@"idAd" value:idAd];
			[cnf updateValue:@"adTime" value:duration];
			[cnf updateValue:@"adFrequency" value:interval];
			
			[cnf release];
		}
		// return;
        [ads release];
	}	
	
}

+ (BOOL) connectedToNetwork
{
    struct sockaddr_in zeroAddress;
    bzero(&zeroAddress, sizeof(zeroAddress));
    zeroAddress.sin_len = sizeof(zeroAddress);
    zeroAddress.sin_family = AF_INET;
	
    SCNetworkReachabilityRef defaultRouteReachability = SCNetworkReachabilityCreateWithAddress(NULL, (struct sockaddr *)&zeroAddress);
    SCNetworkReachabilityFlags flags;
	
    BOOL didRetrieveFlags = SCNetworkReachabilityGetFlags(defaultRouteReachability, &flags);
    CFRelease(defaultRouteReachability);
	
    if (!didRetrieveFlags)
    {
        return 0;
    }
	
    BOOL isReachable = flags & kSCNetworkFlagsReachable;
    BOOL needsConnection = flags & kSCNetworkFlagsConnectionRequired;
    return (isReachable && !needsConnection) ? YES : NO;
}

+ (void) deleteAllFilesInPath: (NSString *)path
{
	
	NSArray *homePaths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *homeDir = [NSString stringWithFormat:@"%@/%@", [homePaths objectAtIndex:0], path];
	
	NSFileManager *fileManager = [NSFileManager defaultManager];
	
	NSError *error=nil;
	
	NSArray *contents = [fileManager contentsOfDirectoryAtPath:homeDir error:nil];
	
	for (NSString *file in contents) {
		BOOL success = [fileManager removeItemAtPath:[homeDir stringByAppendingPathComponent:file] error:&error];
		
		if(!success){
			NSLog(@"couldn't delete file: %@",error);
		}
	}
}

# pragma mark waiting...

+ (void) initWaitView:(UIView *)actualView
{
	
	UIView * waitSubView = [[UIView alloc] init];
	// CGRect posWaitBg = CGRectMake(115, 140, 90, 90);
	CGRect posWaitBg = CGRectMake((actualView.frame.size.width-90)/2, 140, 90, 90);
	waitSubView.frame = posWaitBg;
	waitSubView.hidden = YES;
	waitSubView.hidden = NO;
	waitSubView.tag = 99001;
	waitSubView.backgroundColor = [UIColor clearColor];
	waitSubView.autoresizingMask = (UIViewAutoresizingFlexibleBottomMargin 
								| UIViewAutoresizingFlexibleRightMargin 
								| UIViewAutoresizingFlexibleLeftMargin 
								| UIViewAutoresizingFlexibleTopMargin);
	
	UIImage *waitImg = [UIImage imageNamed:@"loading_bg.png"];
	UIImageView * waitBg = [[UIImageView alloc] initWithImage:waitImg];
	// CGRect 
	//posWaitBg = CGRectMake(115, 140, 90, 90);
	posWaitBg = CGRectMake(0, 0, 90, 90);
	waitBg.frame = posWaitBg;
	// waitBg.hidden = YES;
	[waitSubView addSubview:waitBg];
	[waitBg release];
	
	// CGRect progressFrame = CGRectMake(135, 150, 50, 50);
	CGRect progressFrame = CGRectMake(20, 15, 50, 50);
	UIActivityIndicatorView * waitIndicatorView = [[UIActivityIndicatorView alloc] initWithFrame:progressFrame];
	waitIndicatorView.activityIndicatorViewStyle = UIActivityIndicatorViewStyleWhiteLarge;
	waitIndicatorView.hidesWhenStopped=YES;
	waitIndicatorView.tag = 99002;
	// [waitIndicatorView startAnimating];
	[waitSubView addSubview:waitIndicatorView];
	[waitIndicatorView release];
	
	UILabel * waitLabel = [[UILabel alloc] initWithFrame:CGRectMake(0, 205, 320, 25)];
	progressFrame = CGRectMake(0, 65, 90, 20);
	waitLabel.frame = progressFrame;
	waitLabel.text = @"Loading...";
	waitLabel.backgroundColor = [UIColor clearColor];
	[waitLabel setFont:[UIFont fontWithName:[Functions fontName:@"Arial" fontStyle:@"bold"] size:14]];
	waitLabel.textColor = [Functions colorForHex:@"#FFFFFF"];
	waitLabel.textAlignment = UITextAlignmentCenter; // UITextAlignmentLeft; // UIControlContentHorizontalAlignmentLeft;
	waitLabel.lineBreakMode = UILineBreakModeWordWrap;
	waitLabel.numberOfLines = 0;
	waitLabel.shadowColor = [UIColor grayColor];
	waitLabel.shadowOffset = CGSizeMake(1,1);
	// waitLabel.textColor = [UIColor blueColor];
	// waitLabel.hidden = YES;
	[waitSubView addSubview:waitLabel];
	[waitLabel release];


	[actualView addSubview:waitSubView];
	[waitSubView release];
	
}

+ (void) showAnimated:(UIView *)actualView
{
	/*NSLog(@"start");
	 [waitView startAnimating];
	 waitView.hidden=NO;
	 waitLabel.text = @"Loading...";*/
	UIActivityIndicatorView * tmpIndicator = (UIActivityIndicatorView *)[actualView viewWithTag:99002];
	[tmpIndicator startAnimating];
	UIView * tmpView = (UIView *)[actualView viewWithTag:99001];
	[actualView bringSubviewToFront:tmpView];
	tmpView.hidden = NO;
}

+ (void) stopAnimated:(UIView *)actualView
{
	/*NSLog(@"stop");
	 [waitView stopAnimating];
	 waitView.hidden=YES;
	 waitLabel.text = @"";*/
	UIActivityIndicatorView * tmpIndicator = (UIActivityIndicatorView *)[actualView viewWithTag:99002];
	[tmpIndicator stopAnimating];
	UIView * tmpView = (UIView *)[actualView viewWithTag:99001];
	tmpView.hidden = YES;
}

# pragma mark Hide Tab Bar

+ (BOOL) hidetabbar:(BOOL)hiddenTabBar withViewController:(UIViewController *)viewController
{
	if (!viewController.tabBarController) return NO;
	[UIView beginAnimations:nil context:NULL];
	[UIView setAnimationDuration:0.5];
	
	for(UIView *view in viewController.tabBarController.view.subviews)
	{
		// NSLog(@"%@", view);
		
		if([view isKindOfClass:[UITabBar class]])
		{
			
			if (hiddenTabBar) {
				NSLog(@"1 My view frame: %@", NSStringFromCGRect(view.frame));
				// [view setFrame:CGRectMake(view.frame.origin.x, 411, view.frame.size.width, view.frame.size.height)];
				[view setFrame:CGRectMake(view.frame.origin.x, view.frame.origin.y-49, view.frame.size.width, view.frame.size.height)];
				NSLog(@"2 My view frame: %@", NSStringFromCGRect(view.frame));
			} else {
				NSLog(@"3 My view frame: %@", NSStringFromCGRect(view.frame));
				// [view setFrame:CGRectMake(view.frame.origin.x, 460, view.frame.size.width, view.frame.size.height)];
				[view setFrame:CGRectMake(view.frame.origin.x, view.frame.origin.y+49, view.frame.size.width, view.frame.size.height)];
				NSLog(@"4 My view frame: %@", NSStringFromCGRect(view.frame));
			}
		} else {
			if (hiddenTabBar) {
				// [view setFrame:CGRectMake(view.frame.origin.x, view.frame.origin.y, view.frame.size.width, 411)];
				[view setFrame:CGRectMake(view.frame.origin.x, view.frame.origin.y, view.frame.size.width, view.frame.size.height-49)];
			} else {
				// [view setFrame:CGRectMake(view.frame.origin.x, view.frame.origin.y, view.frame.size.width, 460)];
				[view setFrame:CGRectMake(view.frame.origin.x, view.frame.origin.y, view.frame.size.width, view.frame.size.height+49)];
			}
			
		}
	}
	
	[UIView commitAnimations];
	
	hiddenTabBar = !hiddenTabBar;
	
	return hiddenTabBar;
}

@end
