#import "URLConnection.h"


@implementation URLConnection

@synthesize xmlUrl, xmlDownload, workInProgress;

-(id) initWithUrl:(NSString*)urlData
{
	self = [super init];
	
	if(self)
	{
		self.xmlUrl = urlData;
	}
	
	return self;
}


- (void)setDelegate:(id)new_delegate
{
    m_Delegate = new_delegate;
}	

-(void)downloadData:(id)delegate
{
	m_Delegate = delegate;
	
	NSURLRequest *urlRequest = [NSURLRequest requestWithURL:[NSURL URLWithString:xmlUrl] 
												  cachePolicy:NSURLRequestReturnCacheDataElseLoad timeoutInterval:60.0];
	xmlConnection = [[NSURLConnection alloc] initWithRequest:urlRequest delegate:self];
	
	if(xmlConnection)
	{
		workInProgress = YES;
		xmlData = [[NSMutableData data] retain];
	}
}

-(void)abortDownload
{
	if(workInProgress == YES)
	{
		[xmlConnection cancel];
		[xmlData release];
		workInProgress = NO;
	}
}

- (void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response
{
	
    // this method is called when the server has determined that it
    // has enough information to create the NSURLResponse
    // it can be called multiple times, for example in the case of a
    // redirect, so each time we reset the data.
    // receivedData is declared as a method instance elsewhere
    [xmlData setLength:0];
	
}
- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data
{
    // append the new data to the receivedData
    // receivedData is declared as a method instance elsewhere
    [xmlData appendData:data];	
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error
{
    // release the data object
    [xmlData release];
	
    // inform the user
    // NSLog(@"Connection failed! Error - %@ %@", [error localizedDescription], [[error userInfo] objectForKey:NSErrorFailingURLStringKey]);
    NSLog(@"Error: %@", [error localizedDescription]);
	// Verify that our delegate responds to the InternetImageReady method
	if ([m_Delegate respondsToSelector:@selector(internetUrlError:)])
	{
		// Call the delegate method and pass ourselves along.
		[m_Delegate internetUrlError:self];
	}
	workInProgress = NO;
}

- (void)connectionDidFinishLoading:(NSURLConnection *)connection
{
	if(workInProgress == YES)
	{
		workInProgress = NO;
		// Create the image with the downloaded data
		// UIImage* downloadedImage = [[UIImage alloc] initWithData:xmlData];
		// self.Image = downloadedImage;
		xmlDownload = [[NSMutableData alloc] initWithData:xmlData];
		
		// release the data object
		// [downloadedImage release];
		[xmlData release];
		
		
		// Verify that our delegate responds to the InternetImageReady method
		if ([m_Delegate respondsToSelector:@selector(internetUrlReady:)])
		{
			// Call the delegate method and pass ourselves along.
			[m_Delegate internetUrlReady:self];
		}
	}	
}


- (void)dealloc 
{
    [xmlConnection release];
	[xmlUrl release];
	[xmlDownload release];
	[super dealloc];
}


@end
