//
//  imageInfo.m
//  app_builder
//
//  Created by IVAN BEJARANO on 11/19/10.
//  Copyright 2010 __MyCompanyName__. All rights reserved.
//

#import "imageInfo.h"

@implementation imageInfo

- (id) initWithName:(NSString*)imagename withFolder:(NSString*)foldername withParentFrame:(CGRect)frame
{
    
	NSArray *paths = NSSearchPathForDirectoriesInDomains(
														 NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *docDirectory = [paths objectAtIndex:0];
    docDirectory = [docDirectory stringByAppendingPathComponent:foldername];
    
	NSArray * myArray = [imagename componentsSeparatedByString: @"/"];
	NSString* fileName = (NSString*)[myArray lastObject];
    docDirectory = [docDirectory stringByAppendingPathComponent:fileName];
	// NSLog(@"dir: %@", docDirectory);
    
	BOOL fileExists = [[NSFileManager defaultManager] fileExistsAtPath:docDirectory];
	NSData *imgData; // = [[NSData alloc] init];
	
	if (fileExists) {
		// NSLog(@"exists");
		imgData = [NSData dataWithContentsOfFile:docDirectory];
	} else {
		// self
		// NSLog(@"download");
		NSURL *imgURL     = [NSURL URLWithString:imagename];
		imgData = [NSData dataWithContentsOfURL:imgURL];
		[imgData writeToFile:docDirectory atomically:YES];
	}
    
	// UIImage *img    = [[UIImage alloc] initWithData:imgData];
	// if(self = [super initWithImage:[UIImage imageNamed:imagename]])
	if((self = [super initWithImage:[UIImage imageWithData:imgData]]))
	{
		[self setUserInteractionEnabled:NO];
		
		originalRect = self.frame;
		parentFrame = frame;
		ratio = self.frame.size.width / self.frame.size.height;
		self.autoresizingMask = UIViewAutoresizingFlexibleBottomMargin | UIViewAutoresizingFlexibleTopMargin
        | UIViewAutoresizingFlexibleLeftMargin | UIViewAutoresizingFlexibleRightMargin
        | UIViewAutoresizingFlexibleHeight ;
	}
    return self;
}

- (void) scaleToFitRect:(CGRect)rect
{
	CGFloat newwidth, newheight;
	if(ratio >= 1.0) {
		newwidth = rect.size.width;
		newheight = rect.size.width/ratio;
	}
	else {
		newwidth = rect.size.height*ratio;
		newheight = rect.size.height;
	}
	
	CGRect fullScreenRect = parentFrame; // [[UIScreen mainScreen] applicationFrame];
	CGFloat xoffset = (fullScreenRect.size.width - newwidth) / 2.0;
	CGFloat yoffset = (fullScreenRect.size.height - newheight) / 2.0;
	
	self.frame = CGRectMake(xoffset, yoffset, newwidth, newheight);
}

- (void) centerImage
{
	CGRect fullScreenRect = parentFrame; // [[UIScreen mainScreen] applicationFrame];
	CGFloat xoffset = (fullScreenRect.size.width - self.frame.size.width) / 2.0;
	CGFloat yoffset = (fullScreenRect.size.height - self.frame.size.height) / 2.0;
	self.frame = CGRectMake(xoffset, yoffset, self.frame.size.width, self.frame.size.height);
}

- (void) scaleToOriginalRect
{
	CGRect fullScreenRect = parentFrame; // [[UIScreen mainScreen] applicationFrame];
	CGFloat xoffset = (fullScreenRect.size.width - originalRect.size.width) / 2.0;
	CGFloat yoffset = (fullScreenRect.size.height - originalRect.size.height) / 2.0;
	self.frame = CGRectMake(xoffset, yoffset, originalRect.size.width, originalRect.size.height);
}

- (void) setOffset:(float)xoff:(float)yoff
{
	CGRect r = self.frame;
	r.origin.x = xoff;
	r.origin.y = yoff;
	self.frame = r;
}

- (void) addOffset:(float)xoff:(float)yoff
{
	CGRect r = self.frame;
	r.origin.x += xoff;
	r.origin.y += yoff;
	self.frame = r;
	[self boundOffset];
}

- (void) boundOffset
{
	CGRect r = self.frame;
	CGRect fullScreenRect = parentFrame; // [[UIScreen mainScreen] applicationFrame];
	if(fullScreenRect.size.width > r.size.width) {
		if(r.origin.x < 0) {
			r.origin.x = 0;
		}
		else if(r.origin.x > fullScreenRect.size.width - r.size.width) {
			r.origin.x = fullScreenRect.size.width - r.size.width;
		}
	}
	else {
		if(r.origin.x > 0) {
			r.origin.x = 0;
		}
		else if(r.origin.x < fullScreenRect.size.width - r.size.width) {
			r.origin.x = fullScreenRect.size.width - r.size.width;
		}
	}
	if(fullScreenRect.size.height > r.size.height) {
		if(r.origin.y < 0) {
			r.origin.y = 0;
		}
		else if(r.origin.y > fullScreenRect.size.height - r.size.height) {
			r.origin.y = fullScreenRect.size.height - r.size.height;
		}
	}
	else {
		if(r.origin.y > 0) {
			r.origin.y = 0;
		}
		else if(r.origin.y < fullScreenRect.size.height - r.size.height) {
			r.origin.y = fullScreenRect.size.height - r.size.height;
		}
	}
	self.frame = r;
}

- (float) getRatio {
	return ratio;
}

- (void) dealloc 
{
    [super dealloc];
}

- (void) setParentFrame: (CGRect) frame
{
	parentFrame = frame;
}

@end
