//
//  LineGraph.h
//
//  Created by Robert on 04/10/2009.
//

#import <UIKit/UIKit.h>

typedef enum
{
	LineGraphOrientationUnknown,
	LineGraphOrientationPortrait,            // Device oriented vertically, home button on the bottom
	LineGraphOrientationPortraitUpsideDown,  // Device oriented vertically, home button on the top
	LineGraphOrientationLandscapeLeft,       // Device oriented horizontally, home button on the right
	LineGraphOrientationLandscapeRight,      // Device oriented horizontally, home button on the left
} LineGraphOrientation;

// --------------------------------------------------------
// The Line Object
// --------------------------------------------------------
@interface Line : NSObject
{
@private
	NSString *			name;
	NSMutableArray *	values;
	UIColor *			color;
	float				width;
	Boolean				dashed;
}

@property (nonatomic, readonly) NSString *			name;
@property (nonatomic, readonly) NSMutableArray *	values;
@property (nonatomic, retain)   UIColor *			color;
@property (nonatomic)			float				width;
@property (nonatomic)			Boolean				dashed;

- (id) initWithName:(NSString*)_name AndColor:(UIColor*)_color AndWidth:(float)_width;
- (id) initWithName:(NSString*)_name AndColor:(UIColor*)_color AndWidth:(float)_width SetDashed:(Boolean)_dashed;

- (void) add:(float)_value;
- (void) pop; // remove the first value
- (void) clear;

@end

@interface LineGraph : UIView 
{
@private
	
	// --------------------------------
	// Background Image :
	// will be rendered if not nil.
	// --------------------------------
	UIImage *		 backgroundImage;
	// --------------------------------
	
	// --------------------------------
	// Work out the Graph Values
	// --------------------------------
	NSInteger		 maxNumEntries;
	NSInteger		 minNumEntries;
	NSInteger		 peakEntries;
	float			 maxValue;
	float			 minValue;
	NSInteger		 totalEntries;
	float			 totalValue;
	// --------------------------------
	
	// --------------------------------
	// Data Array :
	// Contains the bar objects.
	// --------------------------------
	NSMutableArray * values;
	
	NSMutableArray * labels;
	
	NSString *		 title;
	NSString *		 verticalAxisName;
	NSString *		 horizontalAxisName;
	
	LineGraphOrientation orientation;
	// --------------------------------
	
	// --------------------------------
	// Graph Indent :
	// How much away from the edges 
	// does the graph get rendered.
	// --------------------------------
	CGSize			indent;
	UIFont *		font;
	UIFont *		fontSmall;
	UIColor	*		fontColor;
	// --------------------------------
	
	// --------------------------------
	// line settings :
	// --------------------------------
	NSInteger		lineWidth;
	UIColor *		lineColor;
	// --------------------------------
	
	// --------------------------------
	// grid data
	// --------------------------------
	NSInteger		gridLineWidth;
	UIColor *		gridLineColor;
	// --------------------------------
	
	// --------------------------------
	// Fill data
	// --------------------------------
	UIColor *		fillColor;
	// --------------------------------
	
	// --------------------------------
	// line values
	// --------------------------------
	float			lineHighlightPercentage;
	UIColor *		lineHighlightColor;
	
	CGPoint			lineShadowOffset;
	UIColor *		lineShadowColor;
	// --------------------------------
	
	
	// --------------------------------
	// Render Flags :
	// Indicate the parts of the line
	// to render.
	// --------------------------------
	Boolean			drawGrid;
	Boolean			drawEdge;
	Boolean			drawLine;
	Boolean			drawFill;
	Boolean			drawScale;
	Boolean			drawKey;
	// -------------------------------
	
}
@property (nonatomic, readonly)	NSInteger count;

@property (nonatomic)			NSInteger maxNumEntries; // max number of points rendered
@property (nonatomic)			NSInteger minNumEntries; // min number of points assumed to be across the graph

@property (nonatomic, retain)	UIImage * backgroundImage;
@property (nonatomic, copy)		NSString * title;
@property (nonatomic, copy)		NSString * verticalAxisName;
@property (nonatomic, copy)		NSString * horizontalAxisName;
@property (nonatomic) LineGraphOrientation orientation;

@property (nonatomic)			CGSize indent;
@property (nonatomic, retain)	UIFont * font;
@property (nonatomic, retain)	UIFont * fontSmall;
@property (nonatomic, retain)	UIColor	* fontColor;
@property (nonatomic)			NSInteger lineWidth;
@property (nonatomic, retain)	UIColor * lineColor;
@property (nonatomic)			NSInteger gridLineWidth;
@property (nonatomic, retain)	UIColor * gridLineColor;
@property (nonatomic, retain)	UIColor * fillColor;
@property (nonatomic)			float lineHighlightPercentage; // 0 - 1 value;
@property (nonatomic, retain)	UIColor * lineHighlightColor;
@property (nonatomic)			CGPoint lineShadowOffset;
@property (nonatomic, retain)	UIColor * lineShadowColor;

@property (nonatomic)			Boolean drawGrid;
@property (nonatomic)			Boolean drawEdge;
@property (nonatomic)			Boolean drawLine;
@property (nonatomic)			Boolean	drawFill;
@property (nonatomic)			Boolean drawScale;
@property (nonatomic)			Boolean drawKey;

// add a line
- (Line*) addLineWithName:(NSString*)_name WithColor:(UIColor*)_color AndWidth:(float)_width;
- (Line*) addLineWithName:(NSString*)_name WithColor:(UIColor*)_color AndWidth:(float)_width SetDashed:(Boolean)_dashed;

// retrieve the line
- (Line*) lineAtIndex:(NSInteger)_index;
- (Line*) lineWithName:(NSString*)_name;

// remove all the current lines
- (void) removeAllLines;
- (void) removeLineAtIndex:(NSInteger)_index;
- (void) removeLineWithName:(NSString*)_name;
- (void) removeLine:(Line*)_line;

// labels
- (void) addLabel:(NSString*)_text;
- (void) clearLabels;


@end
